import random
import time

import sympy as sp
from sympy.abc import x, y


# from sympy.integrals.intpoly import polytope_integrate, Polygon # do testów - coś źle liczy dla
# A = [-3, -1]
# B = [-1, 0]
# C = [-1, 3]
# z = 3*x-3 * y

def calka_podwojna_po_trojkacie():
    a, b = [random.choice(
        [-3, -2, -1, 0, sp.Rational(1, 2), 1, sp.Rational(3, 2), 2, 3]) for _ in
        range(2)]
    c = random.choice([-1, 0, 1, 2])
    while True:
        A = [random.choice([-3, -2, -1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4]) for _ in range(2)]
        B = [random.choice([-3, -2, -1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4]) for _ in range(2)]
        C = [random.choice([-3, -2, -1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4]) for _ in range(2)]
        A, B, C = sorted([A, B, C])
        if (B[0] - A[0]) * (C[1] - A[1]) - (B[1] - A[1]) * (C[0] - A[0]):  # pole trójkąta != 0
            break
    A = [0, -3]
    B = [0, 1]
    C = [-1, -2]
    A, B, C = sorted([A, B, C])
    # a, b, c = sp.Rational(1,2), -2, 1
    # A[0], A[1] = A[1], A[0]
    # B[0], B[1] = B[1], B[0]
    # C[0], C[1] = C[1], C[0]
    # a,b = b,a
    z = a * x + b * y + c
    # print(A, B, C, a, b, c)
    # pol_int = polytope_integrate(Polygon(A, B, C), z)
    # print(pol_int)
    zadanie = f"\\iint\\limits_D\\left({sp.latex(z)}\\right)dx\\,dy"
    if A[0] == B[0]:
        print("pierwszy")
        f = sp.Rational(C[1] - A[1]) / (C[0] - A[0]) * (x - A[0]) + A[1]
        g = sp.Rational(C[1] - B[1]) / (C[0] - B[0]) * (x - B[0]) + B[1]
        if A[1] > B[1]:
            # print("f jest wyżej")
            wynik_x = sp.integrate(z, (y, g, f), (x, A[0], C[0]))
            calka_x = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g)}}}^{{{sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"
        else:
            # print("g jest wyżej")
            wynik_x = sp.integrate(z, (y, f, g), (x, A[0], C[0]))
            calka_x = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f)}}}^{{{sp.latex(g)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"
        # if wynik_x == pol_int:
        #     print("SUKCES")
        # return (
        #     f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$  dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
        #     '$' + calka_x + ' = ' + sp.latex(wynik_x) + '$')
    elif B[0] == C[0]:
        print("drugi")
        f = sp.Rational(C[1] - A[1]) / (C[0] - A[0]) * (x - A[0]) + A[1]
        g = sp.Rational(A[1] - B[1]) / (A[0] - B[0]) * (x - B[0]) + B[1]
        if B[1] > C[1]:
            print("B jest wyżej")
            wynik_x = sp.integrate(z, (y, f, g), (x, A[0], C[0]))
            calka_x = (f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f)}}}^{{{sp.latex(g)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"+
                       f"=\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left( {sp.latex(sp.integrate(z,(y,f,g)))} \\right)dx")
        else:
            print("C jest wyżej")
            wynik_x = sp.integrate(z, (y, g, f), (x, A[0], C[0]))
            calka_x = (f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g)}}}^{{{sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx"+
                       f"=\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left( {sp.latex(sp.integrate(z,(y,g,f)).expand())} \\right)dx")
        # if wynik_x == pol_int:
        #     print("SUKCES")
        # return (
        #     f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$ dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
        #     '$' + calka_x + ' = ' + sp.latex(wynik_x) + '$')
    else:
        print("trzeci")
        f = sp.Rational(C[1] - A[1]) / (C[0] - A[0]) * (x - A[0]) + A[1]
        g = sp.Rational(C[1] - B[1]) / (C[0] - B[0]) * (x - B[0]) + B[1]
        h = sp.Rational(A[1] - B[1]) / (A[0] - B[0]) * (x - B[0]) + B[1]
        # print("f = ", f)
        # print("g = ", g)
        # print("h = ", h)
        if A[1] + (B[0] - A[0]) / (C[0] - A[0]) * (C[1] - A[1]) > B[1]:
            # print("środkowy poniżej")
            wynik_x = sp.integrate(z, (y, h, f), (x, A[0], B[0])) + sp.integrate(z, (y, g, f), (x, B[0], C[0]))
            calka_x = (
                    f"\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(\\int\\limits_{{{sp.latex(h)}}}^{{{sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx + " +
                    f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g)}}}^{{{sp.latex(f)}}}\\left({sp.latex(z)}\\right)dy\\right)dx")
        else:
            # print("środkowy powyżej")
            wynik_x = sp.integrate(z, (y, f, h), (x, A[0], B[0])) + sp.integrate(z, (y, f, g), (x, B[0], C[0]))
            calka_x = (
                    f"\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(\\int\\limits_{{{sp.latex(f)}}}^{{{sp.latex(h)}}}\\left({sp.latex(z)}\\right)dy\\right)dx + " +
                    f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f)}}}^{{{sp.latex(g)}}}\\left({sp.latex(z)}\\right)dy\\right)dx")
        # if wynik_x == pol_int:
        #     print("SUKCES")
    ''' zamieniam x i y miejscami - raczej można to zrobić lepiej '''
    A[0], A[1] = A[1], A[0]
    B[0], B[1] = B[1], B[0]
    C[0], C[1] = C[1], C[0]
    A, B, C = sorted([A, B, C])
    z_y = b * x + a * y + c
    # print(A, B, C)
    # pol_int = polytope_integrate(Polygon(A, B, C), z)
    # print(pol_int)
    # zadanie = f"\\iint\\limits_D\\left({sp.latex(z)}\\right)dx\\,dy"
    if A[0] == B[0]:
        # print("czwarty")
        f = sp.Rational(C[1] - A[1]) / (C[0] - A[0]) * (x - A[0]) + A[1]
        g = sp.Rational(C[1] - B[1]) / (C[0] - B[0]) * (x - B[0]) + B[1]
        if A[1] > B[1]:
            # print("f jest wyżej")
            wynik_y = sp.integrate(z_y, (y, g, f), (x, A[0], C[0]))
            calka_y = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g).replace('x','y')}}}^{{{sp.latex(f).replace('x','y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"
        else:
            # print("g jest wyżej")
            wynik_y = sp.integrate(z_y, (y, f, g), (x, A[0], C[0]))
            calka_y = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f).replace('x','y')}}}^{{{sp.latex(g).replace('x','y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"
        # if wynik_y == pol_int:
        #     print("SUKCES")
        # return (
        #     f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$  dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
        #     '$' + calka_y + ' = ' + sp.latex(wynik_y) + '$')
    elif B[0] == C[0]:
        # print("piąty")
        f = sp.Rational(C[1] - A[1]) / (C[0] - A[0]) * (x - A[0]) + A[1]
        g = sp.Rational(A[1] - B[1]) / (A[0] - B[0]) * (x - B[0]) + B[1]
        if B[1] > C[1]:
            # print("B jest wyżej")
            wynik_y = sp.integrate(z_y, (y, f, g), (x, A[0], C[0]))
            calka_y = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f).replace('x','y')}}}^{{{sp.latex(g).replace('x','y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"
        else:
            # print("C jest wyżej")
            wynik_y = sp.integrate(z_y, (y, g, f), (x, A[0], C[0]))
            calka_y = f"\\int\\limits_{{{A[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g).replace('x','y')}}}^{{{sp.latex(f).replace('x','y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy"
        # if wynik_y == pol_int:
        #     print("SUKCES")
        # return (
        #     f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$ dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
        #     '$' + calka_y + ' = ' + sp.latex(wynik_y) + '$')
    else:
        # print("szósty")
        f = sp.Rational(C[1] - A[1]) / (C[0] - A[0]) * (x - A[0]) + A[1]
        g = sp.Rational(C[1] - B[1]) / (C[0] - B[0]) * (x - B[0]) + B[1]
        h = sp.Rational(A[1] - B[1]) / (A[0] - B[0]) * (x - B[0]) + B[1]
        # print("f = ", f)
        # print("g = ", g)
        # print("h = ", h)
        if A[1] + (B[0] - A[0]) / (C[0] - A[0]) * (C[1] - A[1]) > B[1]:
            # print("środkowy poniżej")
            wynik_y = sp.integrate(z_y, (y, h, f), (x, A[0], B[0])) + sp.integrate(z_y, (y, g, f), (x, B[0], C[0]))
            calka_y = (
                    f"\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(\\int\\limits_{{{sp.latex(h).replace('x','y')}}}^{{{sp.latex(f).replace('x','y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy + " +
                    f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(g).replace('x','y')}}}^{{{sp.latex(f).replace('x','y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy")
        else:
            # print("środkowy powyżej")
            wynik_y = sp.integrate(z_y, (y, f, h), (x, A[0], B[0])) + sp.integrate(z_y, (y, f, g), (x, B[0], C[0]))
            calka_y = (
                    f"\\int\\limits_{{{A[0]}}}^{{{B[0]}}}\\left(\\int\\limits_{{{sp.latex(f).replace('x','y')}}}^{{{sp.latex(h).replace('x','y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy + " +
                    f"\\int\\limits_{{{B[0]}}}^{{{C[0]}}}\\left(\\int\\limits_{{{sp.latex(f).replace('x','y')}}}^{{{sp.latex(g).replace('x','y')}}}\\left({sp.latex(z)}\\right)dx\\right)dy")
        # if wynik_y == pol_int:
        #     print("SUKCES")
    A[0], A[1] = A[1], A[0]
    B[0], B[1] = B[1], B[0]
    C[0], C[1] = C[1], C[0]
    # wynik_y = 1
    if wynik_y != wynik_x:
        print('\n \x1b[1;7;31m' + '!!!Różne wyniki względem osi!!!' + '\x1b[0m \n ')
    return (
        f'Obliczyć ${zadanie}$ gdzie $D$ - trójkąt $ABC$ dla $A=({A[0]},{A[1]}), B=({B[0]},{B[1]}), C=({C[0]},{C[1]})$',
        'Względem $Ox:$ $' + calka_x + ' = ' + sp.latex(wynik_x) + '$ \\\\' +
        'Względem $Oy:$ $' + calka_y + ' = ' + sp.latex(wynik_y) + '$ \\\\')


if __name__ == "__main__":
    start = time.time()
    polecenie, rozwiazanie = calka_podwojna_po_trojkacie()
    print(polecenie, '\n', rozwiazanie)
    print(f'czas generowania: {round(time.time() - start, 3)}s')



import os  # by sprawdzić istnienie pliku gotowców
import pickle  # do ładowania gotowca
import random
import sys  # do czerwonych komunikatów: print('jakis tekst', file=sys.stderr )
import time
from pathlib import Path

import sympy as sp

sciezka = str(Path(__file__).parent)

x = sp.Symbol('x', real=True)


# Todo: Dołożyć rysunki przy polach

def calka_oznaczona(typ: int = 1, gotowiec: bool = False):
    if gotowiec is True:
        if typ == 3:  # 2760/10000 różnych gotowych
            if os.path.isfile(sciezka + '//gotowe//calka_oznaczona_typ_3.pickle'):
                gotowe = pickle.load(open(sciezka + '//gotowe//calka_oznaczona_typ_3.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
        if typ == 4:
            if os.path.isfile(sciezka + '//gotowe//calka_oznaczona_typ_4.pickle'):  # 160/1000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//calka_oznaczona_typ_4.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
    # to nie musi być w else, bo wcześniejsze warunku w przypadku sukcesu konczą funkcje returnem
    if typ == 1:  # parabola i prosta 706/1000 różnych
        while True:
            a, d = [random.choice((-2, -1, 1, 2)) for _ in range(2)]
            while True:
                b, c, e = [random.choice((-3, -2, -1, 0, 1, 2, 3, 4, 5)) for _ in range(3)]
                przeciecia = sp.solve(a * x ** 2 + b * x + c - d * x - e)
                if len(przeciecia) == 2 and all(punkty.is_integer for punkty in przeciecia):
                    break
            break
        F = a * x ** 2 + b * x + c
        G = d * x + e
        calka = abs(sp.integrate(F - G, (x, min(przeciecia), max(przeciecia))))
        posredni = (min(przeciecia) + max(przeciecia)) / 2
        return (f'Obliczyć pole obszaru ograniczonego wykresami krzywych\n'
                f'\t\\[\n'
                f'\t\t f(x) = {sp.latex(F)} \\quad \\textnormal{{ oraz }} \\quad g(x) = {sp.latex(G)} \n'
                f'\t\\]\n',
                f'Pole obszaru to $\\int\\limits_{{{min(przeciecia)}}}^{{{max(przeciecia)}}}'
                f'\\left({sp.latex(F - G) if F.subs(x, posredni) > G.subs(x, posredni) else sp.latex(G - F)}\\right)\\,dx = '
                f' {sp.latex(calka)}$')
    if typ == 2:  # dwie parabole 890/1000 różnych
        while True:
            while True:
                a, d = [random.choice((-2, -1, 1, 2)) for _ in range(2)]
                if a != d:
                    break
            while True:
                b, c, e, f = [random.choice((-3, -2, -1, 0, 1, 2, 3, 4, 5)) for _ in range(4)]
                F = a * x ** 2 + b * x + c
                G = d * x ** 2 + e * x + f
                przeciecia = sp.solve(F - G, x)
                if len(przeciecia) == 2 and all(punkty.is_integer for punkty in przeciecia):
                    break
            break
        calka = abs(sp.integrate(F - G, (x, min(przeciecia), max(przeciecia))))
        # calka = sp.collect(calka, (sp.sin(d * x), sp.cos(d * x)))  # tu czasem źle grupuje sin(x) - to jest jakiś bug - sympy 1.12
        posredni = (min(przeciecia) + max(przeciecia)) / 2
        return (f'Obliczyć pole obszaru ograniczonego wykresami krzywych\n'
                f'\t\\[\n'
                f'\t\t f(x) = {sp.latex(F)} \\quad \\textnormal{{ oraz }} \\quad g(x) = {sp.latex(G)} \n'
                f'\t\\]\n',
                f'Pole obszaru to $\\int\\limits_{{{min(przeciecia)}}}^{{{max(przeciecia)}}}'
                f'\\left({sp.latex(F - G) if F.subs(x, posredni) > G.subs(x, posredni) else sp.latex(G - F)}\\right)\\,dx = '
                f' {sp.latex(calka)}$')
    if typ == 3:  # prosta i wielomian trzeciego stopnia 837/1000 różnych
        while True:
            a, c = [random.choice((-4, -3, -2, -1, 1, 2, 3, 4)) for _ in range(2)]
            b, c, d, e, f = [random.choice((-3, -2, -1, 0, 1, 2, 3, 4, 5)) for _ in range(5)]
            F = a * x + b
            G = c * x ** 3 + d * x ** 2 + e * x + f
            przeciecia = sp.solve(F - G, x)
            if len(przeciecia) == 3 and all((punkty).is_integer for punkty in przeciecia):
                break
        # print(przeciecia, przeciecia[1])
        calka = abs(sp.integrate(abs(F - G), (x, min(przeciecia), max(przeciecia))))
        # calka = sp.collect(calka, (sp.sin(d * x), sp.cos(d * x)))  # tu czasem źle grupuje sin(x) - to jest jakiś bug - sympy 1.12
        posredni1 = (min(przeciecia) + przeciecia[1]) / 2
        posredni2 = (przeciecia[1] + max(przeciecia)) / 2
        # print(posredni1, posredni2, F.subs(x, posredni1) > F.subs(x, posredni1))
        return (f'Obliczyć pole obszaru ograniczonego wykresami krzywych\n'
                f'\t\\[\n'
                f'\t\t f(x) = {sp.latex(F)} \\quad \\textnormal{{ oraz }} \\quad g(x) = {sp.latex(G)} \n'
                f'\t\\]\n',
                f'Pole obszaru to $\\int\\limits_{{{min(przeciecia)}}}^{{{przeciecia[1]}}}'
                f'\\left({sp.latex(F - G) if F.subs(x, posredni1) > G.subs(x, posredni1) else sp.latex(G - F)}\\right)\\,dx + '
                f'\\int\\limits_{{{przeciecia[1]}}}^{{{max(przeciecia)}}}'
                f'\\left({sp.latex(F - G) if F.subs(x, posredni2) > G.subs(x, posredni2) else sp.latex(G - F)}\\right)\\,dx = '
                f' {sp.latex(calka)}$')
    if typ == 4:  # hiperbola i parabola bez wielokrotnych przecieć w jednej ćwiartce
        # Na 1000 losowań wyszło 160 różnych zadań.
        while True:
            a, b = [random.choice((-6, -5, -4, -3, -2, -1, 1, 2, 3, 4, 5, 6)) for _ in range(2)]
            c, d = [random.choice((-6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6)) for _ in range(2)]
            # a, b, c, d = -1, sp.Rational(3 / 2), 5, sp.Rational(9 / 2)
            # a, b, c, d = 1, -sp.Rational(3 / 2), -5, -sp.Rational(9 / 2)

            F = a / x
            G = b * x ** 2 + c * x + d
            przeciecia = sp.solve(F - G, x)
            # print(przeciecia)
            if len(przeciecia) == 3 and all((6 * punkty).is_integer for punkty in przeciecia):
                break
        # calka = abs(sp.integrate((F - G), (x, min(przeciecia), max(przeciecia))))
        calka1 = abs(sp.integrate(F - G, (x, min(przeciecia), przeciecia[1])))
        calka2 = abs(sp.integrate(F - G, (x, przeciecia[1], max(przeciecia))))
        if calka1.is_finite:
            posredni = (min(przeciecia) + przeciecia[1]) / 2
            return (f'Obliczyć pole obszaru ograniczonego wykresami krzywych\n'
                    f'\t\\[\n'
                    f'\t\t f(x) = {sp.latex(F)} \\quad \\textnormal{{ oraz }} \\quad g(x) = {sp.latex(G)} \n'
                    f'\t\\]\n',
                    f'Pole obszaru to $\\int\\limits_{{{min(przeciecia)}}}^{{{przeciecia[1]}}}'
                    f'\\left({sp.latex(F - G) if F.subs(x, posredni) > G.subs(x, posredni) else sp.latex(G - F)}\\right)\\,dx = '
                    f' {sp.latex(calka1)}$'.replace('log', 'ln'))
        else:
            posredni = (max(przeciecia) + przeciecia[1]) / 2
            return (f'Obliczyć pole obszaru ograniczonego wykresami krzywych\n'
                    f'\t\\[\n'
                    f'\t\t f(x) = {sp.latex(F)} \\quad \\textnormal{{ oraz }} \\quad g(x) = {sp.latex(G)} \n'
                    f'\t\\]\n',
                    f'Pole obszaru to $\\int\\limits_{{{przeciecia[1]}}}^{{{max(przeciecia)}}}'
                    f'\\left({sp.latex(F - G) if F.subs(x, posredni) > G.subs(x, posredni) else sp.latex(G - F)}\\right)\\,dx = '
                    f' {sp.latex(calka2)}$'.replace('log', 'ln'))

    if typ == 5:  # hiperbola i parabola z wielokrotnymi przecięciami w jednej ćwiartce. Tylko 24 różne zadania.
        while True:
            # to poniżej daje mało różnych wyników i strasznie długo trwa. Wybrałem z nich tylko te co działają
            # a, b, c, d = [
            #     random.choice((-6, sp.Rational(-11 / 2), -5, sp.Rational(-9 / 2), -4, sp.Rational(-7 / 2), -3,
            #                    sp.Rational(-5 / 2), -2, sp.Rational(-3 / 2), -1, sp.Rational(-1 / 2), 0,
            #                    sp.Rational(1 / 2), 1,
            #                    sp.Rational(3 / 2), 2, sp.Rational(5 / 2), 3, sp.Rational(7 / 2), 4,
            #                    sp.Rational(9 / 2), 5, sp.Rational(11 / 2), sp.Rational(-11 / 3),
            #                    sp.Rational(-9 / 5), sp.Rational(-7 / 3), sp.Rational(-5 / 3), sp.Rational(-3 / 4),
            #                    -sp.Rational(-1 / 3), sp.Rational(1 / 3), sp.Rational(3 / 4), sp.Rational(5 / 4),
            #                    sp.Rational(7 / 3),
            #                    sp.Rational(9 / 4), sp.Rational(11 / 3))) for _ in range(4)]
            a, b, c, d = random.choice(((-3, sp.Rational(1 / 2), 3, sp.Rational(11 / 2)),
                                        (-3, -sp.Rational(1 / 2), 3, -sp.Rational(11 / 2)),
                                        (-2, -sp.Rational(3 / 2), sp.Rational(11 / 2), -6),
                                        (-sp.Rational(3 / 2), -1, sp.Rational(9 / 2), -5),
                                        (-sp.Rational(3 / 2), 1, sp.Rational(9 / 2), 5),
                                        (-1, -sp.Rational(3 / 2), 5, -sp.Rational(9 / 2)),
                                        (-1, -1, sp.Rational(7 / 2), -sp.Rational(7 / 2)),
                                        (-1, 1, sp.Rational(7 / 2), sp.Rational(7 / 2)),
                                        (-1, sp.Rational(3 / 2), 5, sp.Rational(9 / 2)),
                                        (-sp.Rational(1 / 2), -3, sp.Rational(11 / 2), -3),
                                        (-sp.Rational(1 / 2), 3, sp.Rational(11 / 2), 3),
                                        (-sp.Rational(1 / 2), sp.Rational(3 / 4), sp.Rational(5 / 2),
                                         sp.Rational(9 / 4)),
                                        (sp.Rational(1 / 2), -3, -sp.Rational(11 / 2), -3),
                                        (sp.Rational(1 / 2), 3, -sp.Rational(11 / 2), 3),
                                        (sp.Rational(1 / 2), sp.Rational(3 / 4), -sp.Rational(5 / 2),
                                         sp.Rational(9 / 4)),
                                        (1, -sp.Rational(3 / 2), -5, -sp.Rational(9 / 2)),
                                        (1, -1, -sp.Rational(7 / 2), -sp.Rational(7 / 2)),
                                        (1, 1, -sp.Rational(7 / 2), sp.Rational(7 / 2)),
                                        (1, sp.Rational(3 / 2), -5, sp.Rational(9 / 2)),
                                        (sp.Rational(3 / 2), -2, -6, -sp.Rational(11 / 2)),
                                        (sp.Rational(3 / 2), -1, -sp.Rational(9 / 2), -5),
                                        (sp.Rational(3 / 2), 1, -sp.Rational(9 / 2), 5),
                                        (sp.Rational(3 / 2), 2, -6, sp.Rational(11 / 2)),
                                        (2, -sp.Rational(3 / 2), -sp.Rational(11 / 2), -6),
                                        (3, -sp.Rational(1 / 2), -3, -sp.Rational(11 / 2)),
                                        (3, sp.Rational(1 / 2), -3, sp.Rational(11 / 2)),
                                        (-sp.Rational(3 / 4), -sp.Rational(1 / 2), sp.Rational(9 / 4),
                                         -sp.Rational(5 / 2)),
                                        (-sp.Rational(3 / 4), sp.Rational(1 / 2), sp.Rational(9 / 4),
                                         sp.Rational(5 / 2))))
            F = a / x
            G = b * x ** 2 + c * x + d
            przeciecia = sp.solve(F - G, x)
            # print(przeciecia)
            if len(przeciecia) == 3 and all(sp.im(punkty) < 1e-10 for punkty in przeciecia):
                przeciecia = [sp.re(punkty) for punkty in przeciecia]
                przeciecia = sorted(przeciecia)
                # calka = (abs(sp.integrate(F - G, (x, min(przeciecia), przeciecia[1]))) +
                #          abs(sp.integrate(F - G, (x, przeciecia[1], max(przeciecia)))))
                # print(przeciecia, file=sys.stderr)
                if (all((60 * punkty).is_integer for punkty in przeciecia)
                        and (all(sp.sign(punkty) == 1 for punkty in przeciecia)
                             or all(sp.sign(punkty) == -1 for punkty in przeciecia))):
                    break

        calka = (abs(sp.integrate(F - G, (x, min(przeciecia), przeciecia[1]))) +
                 abs(sp.integrate(F - G, (x, przeciecia[1], max(przeciecia)))))
        if calka.is_finite:
            posredni1 = (min(przeciecia) + przeciecia[1]) / 2
            posredni2 = (przeciecia[1] + max(przeciecia)) / 2
            return (f'Obliczyć pole obszaru ograniczonego wykresami krzywych\n'
                    f'\t\\[\n'
                    f'\t\t f(x) = {sp.latex(F)} \\quad \\textnormal{{ oraz }} \\quad g(x) = {sp.latex(G)} \n'
                    f'\t\\]\n',
                    f'Pole obszaru to $\\int\\limits_{{{min(przeciecia)}}}^{{{przeciecia[1]}}}'
                    f'\\left({sp.latex(F - G) if F.subs(x, posredni1) > G.subs(x, posredni1) else sp.latex(G - F)}\\right)\\,dx + '
                    f'\\int\\limits_{{{przeciecia[1]}}}^{{{max(przeciecia)}}}'
                    f'\\left({sp.latex(F - G) if F.subs(x, posredni2) > G.subs(x, posredni2) else sp.latex(G - F)}\\right)\\,dx = '
                    f' {sp.latex(calka)}$'.replace('log', 'ln'))


if __name__ == "__main__":
    start = time.time()
    gotowce = True
    os.chdir('..')  # by wczytywać z gotowca - inaczej problem ze ścieżkami!
    polecenie, rozwiazanie = calka_oznaczona(typ=5, gotowiec=gotowce)
    print(polecenie, '\n', rozwiazanie)
    print(f'czas generowania: {round(time.time() - start, 3)}s')



import os  # by sprawdzić istnienie pliku gotowców
import pickle  # do ładowania gotowca
import random
import sys  # do czerwonych komunikatów: print('jakis tekst', file=sys.stderr )
import time

from pathlib import Path

sciezka = str(Path(__file__).parent)

import sympy as sp

x = sp.Symbol('x', complex=False)


def calka_nieoznaczona(typ: int = 1, gotowiec: bool = False):
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na całkę nieoznaczoną z "ładnymi" rozwiązaniami

    Args:
        typ (int, optional): Typ problemu. Wartości:

            1 - (a * x ** 2 + b * x + c) * g dla g=e^(d x) lub sin(d x) lub cos(d x) lub ln(d x) - 961/1000 różnych,
            2 - (d * x + e) / (a * x ** 2 + b * x + c) - 881/1000 różnych,
            3 - (a * x + b) * sp.sqrt(c * x + d) - 707/1000 różnych.
            4 - a / (c*x+d)**n - 882/1000 różnych

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """
    if gotowiec is True:
        if typ == 3:  # 1462/10000 różnych gotowych
            if os.path.isfile(sciezka + '//gotowe//calka_nieoznaczona_typ_3.pickle'):  # 1000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//calka_nieoznaczona_typ_3.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
    # to nie musi być w else, bo wcześniejsze warunku w przypadku sukcesu konczą funkcje returnem
    if typ == 1:  # 961/1000 różnych
        a, b, c = [random.choice((-3, -2, -1, 0, 1, 2, 3, 4, 5)) for _ in range(3)]
        d = random.choice((1, 2, 3, 4))
        g = random.choice((sp.E ** (d * x), sp.sin(d * x), sp.cos(d * x), sp.ln(d * x)))
        f = (a * x ** 2 + b * x + c) * g
        calka = sp.integrate(f, x)
        # print(calka)
        calka = sp.collect(calka,
                           (sp.sin(d * x),
                            sp.cos(d * x)))  # tu czasem źle grupuje sin(x) - to jest jakiś bug - sympy 1.12

        return (f'Obliczyć całkę\n'
                f'\t\\[\n'
                f'\t\t\\int {sp.latex(f)} \\, dx \n'.replace('log', 'ln') +
                f'\t\\]\n',
                f'$\\int {sp.latex(f)}\\,dx = {sp.latex(calka)} + C$'.replace('log', 'ln'))
    if typ == 2:  # 881/1000 różnych
        tylko_tan = False
        tylko_log = random.choice([True, False, False])
        if not tylko_log:
            tylko_tan = random.choice([True, False, False])
        # if tylko_log: print('tylko log')
        # if tylko_tan: print('tylko tan')
        while True:
            if tylko_log:
                b, c, e = [random.choice((-3, -2, -1, 0, 1, 2, 3, 4, 5)) for _ in range(3)]
                a, d = [random.choice((-2, -1, 1, 2)) for _ in range(2)]
                if (sp.simplify((d * x + e) / (2 * a * x + b))).is_integer and b ** 2 - 4 * a * c < 0:
                    mnoznik = random.choice((-2, -1, 2, 3))
                    d, e = d * mnoznik, e * mnoznik
                    break
            elif tylko_tan:
                b, c = [random.choice((-3, -2, -1, 0, 1, 2, 3, 4, 5)) for _ in range(2)]
                a, e = [random.choice((-2, -1, 1, 2)) for _ in range(2)]
                d = 0
                if b ** 2 - 4 * a * c < 0:
                    break
            else:
                print
                b, c, e = [random.choice((-3, -2, -1, 0, 1, 2, 3, 4, 5)) for _ in range(3)]
                a, d = [random.choice((-2, -1, 1, 2)) for _ in range(2)]
                if b ** 2 - 4 * a * c < 0:
                    break
        f = (d * x + e) / (a * x ** 2 + b * x + c)
        calka = sp.integrate(f, x)

        return (f'Obliczyć całkę\n'
                f'\t\\[\n'
                f'\t\t\\int {sp.latex(f)} \\, dx \n'
                f'\t\\]\n',
                f'$\\int {sp.latex(f)}\\,dx = {sp.latex(calka)} + C$'.replace('log', 'ln'))
    if typ == 3:  # 707/1000 różnych
        while True:
            a, b, c, d = [random.choice((-3, -2, -1, 1, 2, 3, 4, 5)) for _ in range(4)]
            # a, b, c, d, = 2, 2, 5, 5  # to daje wyniki z "Piecewise"
            f = (a * x + b) * sp.sqrt(c * x + d)
            calka = sp.integrate(f, x)
            # print(calka)
            tmp = str(calka)  # tu może jakieś założenie co do zmiennych, by wymusić wyniki rzeczywiste
            if 'I' not in tmp and 'P' not in tmp:  # by wykluczyć zespolone wyniki i cząstkowe z "Piecewise"
                break

        return (f'Obliczyć całkę\n'
                f'\t\\[\n'
                f'\t\t\\int {sp.latex(f)} \\, dx \n'
                f'\t\\]\n',
                f'$\\int {sp.latex(f)}\\,dx = {sp.latex(sp.factor(calka))} + C$')
    if typ == 4:
        a, c, d = [random.choice((-3, -2, -1, 1, 2, 3, 4, 5)) for _ in range(3)]
        n = random.choice((1, 2, 3, 4, 5, 6, 7))
        # a, b, c, d, = 2, 2, 5, 5  # to daje wyniki z "Piecewise"
        f = a / ((c * x + d) ** n)
        calka = sp.integrate(f, x)
        return (f'Obliczyć całkę\n'
                f'\t\\[\n'
                f'\t\t\\int {sp.latex(f)} \\, dx \n'
                f'\t\\]\n',
                f'$\\int {sp.latex(f)}\\,dx = {sp.latex(sp.factor(calka))} + C$')


def calka_wymierna(wlasciwy: bool = False):
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na całkę wymierną z "ładnymi" rozwiązaniami
    Funkcja całkowana to wielomian stopnia co najwyżej dwa plus ułamek prosty pierwszego rodzaju (jeden lub dwa)
    i ułamek prosty drugiego rodzauj (zero lub jeden)

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """
    while True:
        b, c = [random.randint(-4, 4) for i in range(2)]
        if b ** 2 - 4 * c < 0:
            break
    f = (-(int(wlasciwy) - 1) * (random.randint(-2, 2) * x ** 2 + random.randint(-2, 2) * x + random.randint(-2, 2))
         + (-1) ** random.randint(0, 1) * random.randint(1, 3) / (
                 x + (-1) ** random.randint(0, 1) * random.randint(1, 4)) ** random.randint(1, 2)
         + random.randint(0, 1) * random.randint(-2, 3) / (x + random.randint(-4, 4)) ** random.randint(1, 2)
         + random.randint(0, 1) * (random.randint(-2, 2) * x + random.randint(-2, 2)) / (x ** 2 + b * x + c))
    wynik = sp.latex(sp.integrate(f, x)).replace('log', 'ln').replace('operatorname{atan}', 'arctan')
    return (f'Obliczyć całkę\n'
            f'\t\\[\n'
            f'\t\t\\int {sp.latex(sp.cancel(f))} \\, dx \n'
            f'\t\\]\n',
            f'$\\int \\left({sp.latex(f)}\\right)\\,dx = {wynik} + C$')


if __name__ == "__main__":
    start_time = time.time()
    gotowce = True
    os.chdir('..')  # by wczytywać z gotowca - inaczej problem ze ścieżkami!
    # for _ in range(1):
    #     polecenie, rozwiazanie = calka_wymierna(wlasciwy=True)
    # polecenie, rozwiazanie = calka_(typ=3, gotowiec=gotowce)

    for _ in range(10):
        polecenie, rozwiazanie = calka_nieoznaczona(typ=3, gotowiec=gotowce)
        print(polecenie, '\n', rozwiazanie)
    # for _ in range(3):
    #     polecenie, rozwiazanie = calka_nieoznaczona(typ=3)
    print(polecenie, '\n', rozwiazanie)
    print("--- %s seconds ---" % (time.time() - start_time))



import os  # by sprawdzić istnienie pliku gotowców
import pickle  # by pobierać z plików gotowców
import random
from pathlib import Path

import numpy as np
import sympy as sp

sciezka = str(Path(__file__).parent)

# from sympy.abc import x, y, z
x = sp.Symbol('x', real=True)
y = sp.Symbol('y', real=True)
from sympy.abc import z
import time

random.seed()


# Todo: Wzajemne położenie prostych itp, objętość równoległościanów, wysokości opuszczone z wierzchołka
# Todo: Iloczyny skalarne i wektorowe.
# Todo: Równania prostych i płaszczyzn przechodzących przez punkty i równoległych/prostopadłych do wektorów.
# Todo: Odległość prostych skośnych liczona gdy wektor między prostymi jest prostopadły do obu prostych.

def dlugosc(wektor: sp.Matrix) -> sp.Float:
    return sp.sqrt(wektor.dot(wektor))


def rownanie_prostej():
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na wyznaczenie równania prostej przechodzącej
     przez w punkty w R^3 oraz na wyznaczeniu odległości uzyskanej prostej od innego punktu.

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """
    # from sympy.abc import x, y, z
    liczby = (-3, -2, -1, 1, 2, 3, 4, 5)
    while True:
        P_1 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        P_2 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        P_3 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        wektor_kierunkowy = (P_2 - P_1)
        odleglosc = dlugosc((P_2 - P_1).cross(P_3 - P_1)) / dlugosc(P_2 - P_1)
        if round(10 * odleglosc) == 10 * odleglosc and np.count_nonzero(wektor_kierunkowy) >= 2:
            break
    # return (P_1, P_2, P_3), odleglosc
    return (f'Wyznaczyć równanie prostej przechodzącej przez punkty\n'
            f'\t\\[\n'
            f'\t\tP_1 = ({P_1[0]}, {P_1[1]}, {P_1[2]}), \quad P_2 = ({P_2[0]}, {P_2[1]}, {P_2[2]}).\n'
            f'\t\\]\n'
            f'\tObliczyć odległość wyznaczonej prostej od punktu\n'
            f'\t\\[\n'
            f'\t\tP_3 = ({P_3[0]}, {P_3[1]}, {P_3[2]}).\n'
            f'\t\\]',
            f'$l\\colon '
            f' \\frac{{{sp.latex(x - P_1[0])}}}{{{wektor_kierunkowy[0]}}}='
            f' \\frac{{{sp.latex(y - P_1[1])}}}{{{wektor_kierunkowy[1]}}}='
            f' \\frac{{{sp.latex(z - P_1[2])}}}{{{wektor_kierunkowy[2]}}}; \qquad d(P_3,l) = {sp.latex(odleglosc)}$')


def rownanie_plaszczyzny():
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na wyznaczenie równania płaszczyzny przechodzącej
     przez 3 punkty w R^3 oraz na wyznaczeniu odległości uzyskanej płaszczyzny od innego punktu.

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """
    # from sympy.abc import x, y, z
    liczby = (-3, -2, -1, 0, 1, 2, 3, 4, 5)
    while True:
        P_1 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        P_2 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        P_3 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        P_4 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        wektor_normalny = (P_2 - P_1).cross(P_3 - P_1)
        wyraz_wolny_plaszczyzny = - wektor_normalny.dot(P_1)
        odleglosc = abs(wektor_normalny.dot(P_4) + wyraz_wolny_plaszczyzny) / dlugosc(wektor_normalny)
        if round(10 * odleglosc) == 10 * odleglosc and np.count_nonzero(wektor_normalny) >= 2:
            break
    return (f'Wyznaczyć równanie płaszczyzny przechodzącej przez punkty\n'
            f'\t\\[\n'
            f'\t\tP_1 = ({P_1[0]}, {P_1[1]}, {P_1[2]}), \quad P_2 = ({P_2[0]}, {P_2[1]}, {P_2[2]}),'
            f' \quad P_3 = ({P_3[0]}, {P_3[1]}, {P_3[2]}).\n'
            f'\t\\]\n'
            f'\tObliczyć odległość wyznaczonej płaszczyzny od punktu\n'
            f'\t\\[\n'
            f'\t\tP_4 = ({P_4[0]}, {P_4[1]}, {P_4[2]}).\n'
            f'\t\\]',
            f'$\pi\\colon '
            f'{sp.latex(wektor_normalny[0] * x + wektor_normalny[1] * y + wektor_normalny[2] * z + wyraz_wolny_plaszczyzny)}=0;'
            f' \qquad d(P_4,\pi) = {sp.latex(odleglosc)}$')


def odleglosc_prostych_skosnych(gotowiec: bool = False):
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na wyznaczenie odległości prostych skośnych w R^3

    Args:
        gotowiec (bool, optional): Czy zadanie ma być pobierane z gotowego zestawu. Defaults to False.
            W gotowcu jest 1000 różnych (na 1000 losowań)
    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """

    if (gotowiec is True
            and os.path.isfile(sciezka + '//gotowe//odleglosc_prostych_skosnych.pickle')):  # 10000 różnych gotowych
        gotowe = pickle.load(open(sciezka + '//gotowe//odleglosc_prostych_skosnych.pickle', 'rb'))
        return gotowe[random.randint(0, len(gotowe)) - 1]
    else:
        while True:
            liczby = (-3, -2, -1, 1, 2, 3, 4, 5)
            k_1 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
            P_1 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
            k_2 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
            P_2 = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
            # k_1 = sp.Matrix([2,1,2])
            # P_1 = sp.Matrix([-2,-3,-1])
            # k_2 = sp.Matrix([4,3,2])
            # P_2 = sp.Matrix([2,-3,-2])
            wektor_normalny = k_1.cross(k_2)
            wyraz_wolny_plaszczyzny = - wektor_normalny.dot(P_2)
            # odleglosc1 = abs(wektor_normalny.dot(P_1) + wyraz_wolny_plaszczyzny) / dlugosc(wektor_normalny)
            if dlugosc(wektor_normalny) > 0.0001:
                odleglosc2 = abs(k_1.dot(k_2.cross(P_1 - P_2))) / dlugosc(wektor_normalny)
                # print(odleglosc2)
                # to można prościej zrobić płaszczyznami
                # równanie płaszczyzny można zrobić tak:
                # n.dot(sp.Matrix((x, y, z)) - sp.Matrix((2, 3, 4)))
                # a jeszcze prościej można zrobić wektorem łączącym proste. On musi być prostopadły do obu
                # https://brilliant.org/wiki/3d-coordinate-geometry-skew-lines/
                parametr_t = ((k_2.dot(k_2)) * k_1 - (k_1.dot(k_2)) * k_2).dot(P_2 - P_1) / (
                    k_1.cross(k_2).dot(k_1.cross(k_2)))
                Punkt_1 = k_1 * parametr_t + P_1
                parametr_u = ((k_1.dot(k_1) * k_2) - (k_1.dot(k_2)) * k_1).dot(P_1 - P_2) / (
                    k_1.cross(k_2).dot(k_1.cross(k_2)))
                Punkt_2 = k_2 * parametr_u + P_2
                if (round(10 * odleglosc2) == 10 * odleglosc2
                        and odleglosc2 > 0.01
                        and round(2 * parametr_t) == 2 * parametr_t
                        and round(2 * parametr_u) == 2 * parametr_u):
                    break
        # return (k_1, P_1, k_2, P_2), odleglosc2, parametr_t, parametr_u, Punkt_1, Punkt_2, dlugosc(
        #     Punkt_1 - Punkt_2), sp.N(odleglosc2 - dlugosc(Punkt_2 - Punkt_1))
        return (f'Obliczyć odległość prostych skośnych\n'
                f'\t\\[\n'
                f'\t\tl_1\\colon \\frac{{{sp.latex(x - P_1[0])}}}{{{k_1[0]}}}='
                f'\\frac{{{sp.latex(y - P_1[1])}}}{{{k_1[1]}}}=\\frac{{{sp.latex(z - P_1[2])}}}{{{k_1[2]}}}, \\quad \n'
                f'\t\tl_2\\colon \\frac{{{sp.latex(x - P_2[0])}}}{{{k_2[0]}}}='
                f'\\frac{{{sp.latex(y - P_2[1])}}}{{{k_2[1]}}}=\\frac{{{sp.latex(z - P_2[2])}}}{{{k_2[2]}}}.\n'
                f'\t\\]'
                f'\tWyznaczyć punkty realizujące minimalną odległość.',
                f'Płaszczyzna zawierająca $l_2$ i równoległa do $l_1$ to $\\pi\\colon '
                f'{sp.latex(wektor_normalny[0] * x + wektor_normalny[1] * y + wektor_normalny[2] * z + wyraz_wolny_plaszczyzny)}=0$;\\\\'
                f'$d(l_1,l_2)={sp.latex(odleglosc2)};$\\quad '
                f'Punkty realizujące minimalną odległość to:\ '
                f'$ P_3=({sp.latex(Punkt_1[0])},{sp.latex(Punkt_1[1])},{sp.latex(Punkt_1[2])}),\ '
                f'P_4=({sp.latex(Punkt_2[0])},{sp.latex(Punkt_2[1])},{sp.latex(Punkt_2[2])})$.\n')


def punkt_symetryczny_do_plaszczyzny():  # na 50000 losowań było 6862 róznych wyników
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na wyznaczenie punktu symetrycznego względem płaszczyzny.

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """
    liczby = (-3, -2, -1, 1, 2, 3, 4, 5)
    # from sympy.abc import x, y, z
    while True:
        '''Parametry płaszczyzny Ax+By+Cz+D=0'''
        # plaszczyzna = [sp.Rational(random.choice(liczby)) for _ in range(4)]
        A, B, C, D = [sp.Rational(random.choice(liczby)) for _ in range(4)]
        '''Współrzędne punktu do odbicia P=(P_x, P_y, P_z)'''
        # punkt = list([random.choice(liczby) for _ in range(3)])
        P_x, P_y, P_z = list([random.choice(liczby) for _ in range(3)])
        t_przeciecia = (
                (-D - A * P_x - B * P_y - C * P_z) /
                (A ** 2 + B ** 2 + C ** 2))
        if t_przeciecia.is_integer is True and t_przeciecia != 0 and t_przeciecia != 1 and t_przeciecia != -1:
            # punkt_przeciecia = [0] * 3
            break
    Pp_x = P_x + A * t_przeciecia
    Pp_y = P_y + B * t_przeciecia
    Pp_z = P_z + C * t_przeciecia
    # punkt_symetryczny = [None] * 3
    Ps_x = 2 * Pp_x - P_x
    Ps_y = 2 * Pp_y - P_y
    Ps_z = 2 * Pp_z - P_z
    pi = A * x + B * y + C * z + D
    return (f'Wyznaczyć punkt symetryczny do punktu\n'
            f'\t\\[\n'
            f'\t\tP = ({P_x}, {P_y}, {P_z})\n'
            f'\t\\]\n'
            f'\twzględem płaszczyzny\n'
            f'\t\\[\n'
            f'\t\t\\pi\\colon {sp.latex(pi)}  = 0.\n'
            f'\t\\]',
            f'Prosta prostopadła: $'
            f' \\frac{{{sp.latex(x - P_x)}}}{{{A}}}='
            f' \\frac{{{sp.latex(y - P_y)}}}{{{B}}}='
            f' \\frac{{{sp.latex(z - P_z)}}}{{{C}}}= t,$ \\quad $t_p={t_przeciecia}$ \\\\\n'
            f'Punkt przecięcia to: $P_p ='
            f'({Pp_x},{Pp_y},{Pp_z}),$ \\quad \n'
            f'Punkt symetryczny to: $P_s = '
            f'({Ps_x},{Ps_y},{Ps_z})$')


def punkt_symetryczny_do_prostej():  # na 50000 losowań było 37341 różnych wyników
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na wyznaczenie punktu symetrycznego względem płaszczyzny.

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """
    liczby = (-3, -2, -1, 1, 2, 3, 4, 5)
    # from sympy.abc import x, y, z
    while True:
        '''Parametry prostej w postaci kierunkowej (x-x_0)/A = (y-y_0)/B = (z-z_0)/C'''
        A, B, C, x_0, y_0, z_0 = [sp.Rational(random.choice(liczby)) for _ in range(6)]
        # A, B, C, x_0, y_0, z_0 = [sp.Rational(2), 1, 1, 5, 2, 5]
        '''Współrzędne punktu do odbicia P=(P_x, P_y, P_z)'''
        P_x, P_y, P_z = [random.choice(liczby) for _ in range(3)]
        # P_x, P_y, P_z = [2, -2, -3]
        t_przeciecia = (A * (P_x - x_0) + B * (P_y - y_0) + C * (P_z - z_0)) / (A ** 2 + B ** 2 + C ** 2)
        # print(A, B, C, x_0, y_0, z_0, P_x, P_y, P_z, t_przeciecia, type(t_przeciecia), t_przeciecia.is_integer)
        if t_przeciecia.is_integer is True and t_przeciecia != 0 and t_przeciecia != 1 and t_przeciecia != -1:
            break
    # punkt_przeciecia = [0] * 3
    Pp_x = x_0 + A * t_przeciecia
    Pp_y = y_0 + B * t_przeciecia
    Pp_z = z_0 + C * t_przeciecia
    # punkt_symetryczny = [None] * 3
    Ps_x = 2 * Pp_x - P_x
    Ps_y = 2 * Pp_y - P_y
    Ps_z = 2 * Pp_z - P_z
    l = (f'\\frac{{{sp.latex(x - x_0)}}}{{{A}}}='
         f' \\frac{{{sp.latex(y - y_0)}}}{{{B}}}='
         f' \\frac{{{sp.latex(z - z_0)}}}{{{C}}}')
    pi = (A * x + B * y + C * z - A * P_x - B * P_y - C * P_z)

    return (f'Wyznaczyć punkt symetryczny do punktu\n'
            f'\t\\[\n'
            f'\t\tP = ({P_x}, {P_y}, {P_z})\n'
            f'\t\\]\n'
            f'\twzględem prostej\n'
            f'\t\\[\n'
            f'\t\tl\\colon {l}.\n'
            f'\t\\]',
            f'Płaszczyzna prostopadła: $\\pi\\colon '
            f'{sp.latex(pi)} = 0, \\quad t_p={t_przeciecia}$ \\\\\n'
            f'\t\t\t\tPunkt przecięcia to: $P_p ='
            f'({Pp_x},{Pp_y},{Pp_z}),$ \\quad \n'
            f'\t\t\t\tPunkt symetryczny to: $P_s = '
            f'({Ps_x},{Ps_y},{Ps_z})$')


def katy_w_trojkacie(prosty: bool = True, calkowite: bool = True):
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na wyznaczenie kątów wewnętrznych trójkąta w R^3.

    Args:
        prosty (bool, optional): Czy trójkąt ma mieć kąt prosty. Defaults to True.
        calkowite (bool, optional): Czy kąty mają być całkowite. Defaults to True.

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """
    liczby = (-3, -2, -1, 0, 1, 2, 3, 4, 5)
    while True:
        while True:
            A_x, A_y, A_z = [sp.Rational(random.choice(liczby)) for _ in range(3)]
            B_x, B_y, B_z = [sp.Rational(random.choice(liczby)) for _ in range(3)]
            C_x, C_y, C_z = [sp.Rational(random.choice(liczby)) for _ in range(3)]

            alpha = sp.acos(((B_x - A_x) * (C_x - A_x) + (B_y - A_y) * (C_y - A_y) + (B_z - A_z) * (C_z - A_z))
                            / (sp.sqrt((B_x - A_x) ** 2 + (B_y - A_y) ** 2 + (B_z - A_z) ** 2)
                               * sp.sqrt((C_x - A_x) ** 2 + (C_y - A_y) ** 2 + (C_z - A_z) ** 2))).evalf()
            beta = sp.acos(((C_x - B_x) * (A_x - B_x) + (C_y - B_y) * (A_y - B_y) + (C_z - B_z) * (A_z - B_z))
                           / (sp.sqrt((C_x - B_x) ** 2 + (C_y - B_y) ** 2 + (C_z - B_z) ** 2)
                              * sp.sqrt((A_x - B_x) ** 2 + (A_y - B_y) ** 2 + (A_z - B_z) ** 2))).evalf()
            gamma = sp.acos(((A_x - C_x) * (B_x - C_x) + (A_y - C_y) * (B_y - C_y) + (A_z - C_z) * (B_z - C_z))
                            / (sp.sqrt((A_x - C_x) ** 2 + (A_y - C_y) ** 2 + (A_z - C_z) ** 2)
                               * sp.sqrt((B_x - C_x) ** 2 + (B_y - C_y) ** 2 + (B_z - C_z) ** 2))).evalf()
            # print(alpha, beta, gamma)
            if alpha != sp.nan and beta != sp.nan and gamma != sp.nan:
                break
        # print(prosty)
        if calkowite is not True:
            if ((prosty is not True)
                    or abs(alpha - sp.pi / 2) < 10 ** (-10)
                    or abs(beta - sp.pi / 2) < 10 ** (-10)
                    or abs(gamma - sp.pi / 2) < 10 ** (-10)):
                break
        else:
            if (((abs(alpha - sp.pi / 2) < 10 ** (-10)
                  or abs(alpha - sp.pi / 3) < 10 ** (-10)
                  or abs(alpha - sp.pi / 4) < 10 ** (-10)
                  or abs(alpha - sp.pi / 6) < 10 ** (-10))
                 and (abs(beta - sp.pi / 2) < 10 ** (-10)
                      or abs(beta - sp.pi / 3) < 10 ** (-10)
                      or abs(beta - sp.pi / 4) < 10 ** (-10)
                      or abs(beta - sp.pi / 6) < 10 ** (-10))
                 and (abs(gamma - sp.pi / 2) < 10 ** (-10)
                      or abs(gamma - sp.pi / 3) < 10 ** (-10)
                      or abs(gamma - sp.pi / 4) < 10 ** (-10)
                      or abs(gamma - sp.pi / 6) < 10 ** (-10)))):
                break

    # print(A_x, A_y, A_z, B_x, B_y, B_z, C_x, C_y, C_z)
    alpha = sp.re((alpha * 180 / sp.pi).evalf(10))
    beta = sp.re((beta * 180 / sp.pi).evalf(10))
    gamma = sp.re((gamma * 180 / sp.pi).evalf(10))
    return (f'Wyznaczyć miary kątów wewnętrznych trójkąta $ABC,$ gdzie\n'
            f'\t\\[\n'
            f'\t\tA = ({A_x}, {A_y}, {A_z}),\\ B = ({B_x}, {B_y}, {B_z}),\\ C = ({C_x}, {C_y}, {C_z})\n'
            f'\t\\]\n'
            f'\tSprawdzić, czy sumują się do $180^{{\\circ}}.$\\\\',
            f'$\\alpha ' +
            str('= ' if round(alpha, 1) == round(alpha, 5) else '\\approx ') +
            f'{alpha:.2f}^{{\\circ}},\\  '
            f'\\beta ' +
            str('= ' if round(beta, 1) == round(beta, 5) else '\\approx ') +
            f'{beta:.2f}^{{\\circ}},\\  '
            f'\\gamma ' +
            str('= ' if round(gamma, 1) == round(gamma, 5) else '\\approx ') +
            f'{gamma:.2f}^{{\\circ}}.$')


def pole_trojkata(calkowite: bool = False):
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na wyznaczenie pola trójkąta w R^3 oraz jednej z wysokości.

    Args:
        calkowite (bool, optional): Czy pole i wysokości mają być całkowite lub podzielne przez dwa. Defaults to True.

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.
    """
    liczby = (-3, -2, -1, 1, 2, 3, 4, 5)
    wierzcholek = random.choice(('A', 'B', 'C'))
    while True:
        A = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        B = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        C = sp.Matrix([sp.Rational(random.choice(liczby)) for _ in range(3)])
        AB = A - B
        AC = A - C
        BC = B - C
        # BA = B - A
        CA = C - A
        # CB = C - B
        pole = sp.sqrt((AB.cross(AC)).dot(AB.cross(AC))) / 2
        wysokosc = 0  # niepotrzebne w zasadzie, ale bez tego pycharm zgłasza potencjalny błąd nieprzypisania wartości
        match wierzcholek:
            case 'A':
                wysokosc = 2 * pole / sp.sqrt(BC.dot(BC))
            case 'B':
                wysokosc = 2 * pole / sp.sqrt(CA.dot(CA))
            case 'C':
                wysokosc = 2 * pole / sp.sqrt(AB.dot(AB))
        if (pole != 0
                and pole < 10
                and wysokosc < 10
                and ((round(wysokosc) == wysokosc and round(2 * pole) == 2 * pole) or not calkowite)):
            break

    return (
        f'Wyznaczyć pole trójkąta $ABC$ oraz długość wysokości opuszczonej z wierzchołka {wierzcholek} dla\n'
        f'\t\\[\n'
        f'\t\tA = ({A[0]}, {A[1]}, {A[2]}),\\ B = ({B[0]}, {B[1]}, {B[2]}),\\ C = ({C[0]}, {C[1]}, {C[2]})\n'
        f'\t\\]',
        f'$P={sp.latex(pole)},\\ \\ h_{wierzcholek}={sp.latex(wysokosc)}$')
    # A.T, B.T, C.T, calkowite, pole, wysokosc, wierzcholek


def plaszczyzna_styczna():
    typ = random.choice([i for i in range(6)])
    while True:
        a, b, c, d, e, f, g, h, i, j, k, l = [random.choice([n - 4 for n in range(10)]) for _ in range(12)]
        if typ == 0:
            z = a * (x - b) ** 2 * (y - c) ** 2 + d * (x - e) * (y - f)
        elif typ == 1:
            z = a * (x) ** 2 + b * (y) ** 2 + d * x ** 3 * y + e * x * y ** 3 + f * x ** 2 * y
        elif typ == 2:
            z = a * x / y + b * y / x + c * x + d * y + e / x + f / y
        elif typ == 3:
            z = (f * x + g * y) * sp.exp(a * x ** 2 + b * x + c * y ** 2 + d * y + e)
        elif typ == 4:
            z = sp.sqrt(c ** 2 - a * x ** 2 - b * y ** 2)
        elif typ == 5:
            z = sp.sqrt(c ** 2 - a * x ** 2 - b * y ** 2) / (d * x + e * y)
        P = [random.choice([n - 4 for n in range(10)]) for _ in range(2)]
        try:
            z_x = sp.diff(z, x).subs({x: P[0], y: P[1]})
            z_y = sp.diff(z, y).subs({x: P[0], y: P[1]})
            # print(z.is_real, z, z_x, z_y)
            if ((z.subs({x: P[0], y: P[1]})).is_real and z.subs({x: P[0], y: P[1]}) < 10
                    and 0 < abs(z_x) < 10 and 0 < abs(z_y) < 10 and z_x.is_Rational and z_y.is_Rational):
                break
        except:
            pass
    return (
        f'Wyznaczyć płaszczyznę styczną do powierzchni $$f(x,y)={sp.latex(z)}$$ w punkcie $P=({P[0]},{P[1]},f({P[0]},{P[1]})).$',
        f'$z = {sp.latex(z_x * (x - P[0]) + z_y * (y - P[1]) + z.subs({x: P[0], y: P[1]}))}$')


if __name__ == "__main__":  # to się uruchamia tylko, gdy plik jest uruchamiany jako program, a nie ładowany jako moduł
    os.chdir('..')  # by wczytywać z gotowca - inaczej problem ze ścieżkami!
    start_time = time.time()
    # polecenie, rozwiazanie = rownanie_prostej()
    # polecenie, rozwiazanie = rownanie_plaszczyzny()
    # polecenie, rozwiazanie = odleglosc_prostych_skosnych(gotowiec=True)
    # polecenie, rozwiazanie = punkt_symetryczny_do_plaszczyzny()
    # polecenie, rozwiazanie = punkt_symetryczny_do_prostej()
    # polecenie, rozwiazanie = katy_w_trojkacie(prosty=random.choice((False, True)),
    #                                           calkowite=random.choice((False, True)))
    # polecenie, rozwiazanie = pole_trojkata(calkowite=random.choice((True, False)))
    polecenie, rozwiazanie = plaszczyzna_styczna()
    print(polecenie, '\n', rozwiazanie)
    print("--- %s seconds ---" % (time.time() - start_time))
    # pass



import os  # by sprawdzić istnienie pliku gotowców
import pickle  # do ładowania gotowca
import random
import sys  # do czerwonych komunikatów: print('jakis tekst', file=sys.stderr )
import time
from pathlib import Path

import sympy as sp

sciezka = str(Path(__file__).parent)


# Todo: Maksymalne przedziały monotoniczności domknięte
# Todo: W monotonicznościach nie używać sum

def styczna_normalna(typ: int = 1):
    if typ == 1:  # parabola przez prostą
        while True:
            x = sp.Symbol('x', real=True)
            x_0 = random.choice((-2, -1, 2, 3))
            a, c, d, e = [random.choice((-2, -1, 1, 2, 3, 4)) for _ in range(4)]
            b = random.choice((-2, -1, 0, 1, 2, 3, 4))
            f = ((a * x ** 2 + b * x + c) / (d * x + e)) ** (random.choice((-1, 1)))
            # print(a,b,c,d)
            f_prim = f.diff(x)
            try:
                f_x_0 = f.subs(x, x_0)
                f_prim_x_0 = f_prim.subs(x, x_0)
                if (f_prim_x_0 != 0
                        and f_x_0 != 0
                        and (2 * f_prim_x_0).is_integer
                        and (2 * f_x_0).is_integer
                        and sp.Abs(f_prim_x_0) < 10
                        and sp.Abs(f_x_0) < 10):
                    break
            except ValueError:
                pass
        # print(f_x_0, f_prim_x_0)
        return (f'Wyznaczyć równanie stycznej i normalnej do wykresu funkcji\n'
                f'\t\\[\n'
                f'\t\tf(x) = {sp.latex(f)}\n'
                f'\t\\]\n'
                f' w punkcie $x_0 = {sp.latex(x_0)}.$',
                f'Styczna: $y = {sp.latex(f_prim_x_0 * (x - x_0) + f_x_0)},$ \\quad\n'
                f'Normalna: $y = {sp.latex(-1 / f_prim_x_0 * (x - x_0) + f_x_0)}$')
    if typ == 2:  # pierwiastek z paraboli przez prostą
        potega = random.choice((0, 1))  # by w do drugim był mianownik
        x_0 = random.choice((-2, -1, 2, 3))  # by nie wychodziło za często sqrt(1)
        wykladnicza = random.choice((0, 1))
        while True:
            x = sp.Symbol('x', real=True)
            a, c, d, e = [random.choice((-2, -1, 1, 2, 3, 4)) for _ in range(4)]
            b = random.choice((-2, -1, 0, 1, 2, 3, 4))
            f = (sp.sqrt(a * x ** 2 + b * x + c) if wykladnicza == 0 else sp.E ** (a * x ** 2 + b * x + c)) / (
                        d * x + e) ** (potega)
            # print(a,b,c,d)
            f_prim = f.diff(x)
            try:
                f_x_0 = f.subs(x, x_0)
                f_prim_x_0 = f_prim.subs(x, x_0)
                if (f_prim_x_0 != 0
                        and f_x_0 != 0
                        and (2 * f_prim_x_0).is_integer
                        and (2 * f_x_0).is_integer
                        and sp.Abs(f_prim_x_0) < 10
                        and sp.Abs(f_x_0) < 10):
                    break
            except ValueError:
                pass
        # print(f_x_0, f_prim_x_0)
        return (f'Wyznaczyć równanie stycznej i normalnej do wykresu funkcji\n'
                f'\t\\[\n'
                f'\t\tf(x) = {sp.latex(f)}\n'
                f'\t\\]\n'
                f' w punkcie $x_0 = {sp.latex(x_0)}.$',
                f'Styczna: $y = {sp.latex(f_prim_x_0 * (x - x_0) + f_x_0)},$ \\quad\n'
                f'Normalna: $y = {sp.latex(-1 / f_prim_x_0 * (x - x_0) + f_x_0)}$')


def monotonicznosc(typ: int = 1, gotowiec: bool = False):
    """
    Generuje zadanie (w formacie LaTeX wraz z rozwiązaniem) na badanie monotoniczności i ekstremów dla różnych funkcji.

    Args:
        typ (int, optional, defaults to 1):
            1 - parabola przez prostą
            2 - prosta przez parabolę, w gotowcu są 365 różne (na 1000 losowań)
            3 - wielomian 3-go stopnia przez prostą
        gotowiec (bool, optional, defaults to False): Czy zadanie ma być pobierane z gotowego zestawu.

    Returns:
        tuple: Zawiera polecenie oraz jego odpowiednie rozwiązanie.ion of the results.
    """
    if gotowiec is True:
        if typ == 2:
            if os.path.isfile(sciezka + '//gotowe//monotonicznosc_typ_2.pickle'):  # 1000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//monotonicznosc_typ_2.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
        if typ == 3:
            if os.path.isfile(sciezka + '//gotowe//monotonicznosc_typ_3.pickle'):  # 356/1000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//monotonicznosc_typ_3.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
    # to nie musi być w else, bo wcześniejsze warunku w przypadku sukcesu konczą funkcje returnem
    if typ == 1:  # parabola przez prostą
        while True:
            x = sp.Symbol('x', real=True)
            a, c, d, e = [random.choice((-2, -1, 1, 2, 3, 4)) for _ in range(4)]
            b = random.choice((-2, -1, 0, 1, 2, 3, 4))
            f = (a * x ** 2 + b * x + c) / (d * x + e)
            # print(a,b,c,d)
            f_prim = f.diff(x)
            f_bis = f_prim.diff(x)
            krytyczne = sp.solve(f_prim, x)

            if (all([el.is_real for el in krytyczne])
                    and all([10 * el == int(10 * el) for el in krytyczne])
                    and len(krytyczne) > 0):
                break
                # print(f'rozwiazania {krytyczne}')
                # print(sp.latex(f))
        rosnaca = sp.solve(f_prim > 0, x)
        malejaca = sp.solve(f_prim < 0, x)
        if a / d < 0:
            return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                    f'\t\\[\n'
                    f'\t\tf(x)={sp.latex(f)} \n'
                    f'\t\\]\n',
                    f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                    # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                    f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                    f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                    f'x \\in \\left({sp.latex(rosnaca.args[0].args[1])},{sp.latex(rosnaca.args[2].args[1])}  \\right)'
                    f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(rosnaca.args[2].args[1])},{sp.latex(rosnaca.args[1].args[1])}  \\right) $\\newline\n'
                    f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                    f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                    f'x \\in \\left( -\\infty, {sp.latex(malejaca.args[1].args[1])}  \\right)'
                    f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(malejaca.args[0].args[0])}, \\infty  \\right) $\\newline\n'
                    f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                    f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                    f' \\implies ' +
                    ('f_{\\min}' if f_bis.subs(x, krytyczne[0]) > 0 else 'f_{\\max}') +
                    f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))} \\textnormal{{ oraz }} ' +
                    ('f_{\\min}' if f_bis.subs(x, krytyczne[1]) > 0 else 'f_{\\max}') +
                    f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$')
        else:
            return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                    f'\t\\[\n'
                    f'\t\tf(x)={sp.latex(f)} \n'
                    f'\t\\]\n',
                    f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                    # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                    f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                    f'{sp.latex(rosnaca)}\\implies f(x) \\textnormal{{ rośnie dla }} '
                    f'x \\in \\left( -\\infty, {sp.latex(rosnaca.args[1].args[1])}  \\right)'
                    f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(rosnaca.args[0].args[0])}, \\infty  \\right)$\\newline\n'
                    f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                    f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                    f'x \\in \\left({sp.latex(malejaca.args[0].args[1])},{sp.latex(malejaca.args[2].args[1])}  \\right)'
                    f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(malejaca.args[2].args[1])},{sp.latex(malejaca.args[1].args[1])}  \\right)$\\newline\n'
                    f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                    f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                    f' \\implies ' +
                    ('f_{\\min}' if f_bis.subs(x, krytyczne[0]) > 0 else 'f_{\\max}') +
                    f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))} \\textnormal{{ oraz }} ' +
                    ('f_{\\min}' if f_bis.subs(x, krytyczne[1]) > 0 else 'f_{\\max}') +
                    f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$')
    if typ == 2:  # prosta przez parabolę - trochę trwa więce jest gotowiec z '' zadaniami.
        ciekawe = random.choice((0, 1))  # co drugie powinno być ciekawe tzn. rozbijane na kilka przedziałów
        while True:
            x = sp.Symbol('x', real=True)
            a, c, d, e = [random.choice((-2, -1, 1, 2, 3, 4)) for _ in range(4)]
            b = random.choice((-2, -1, 0, 1, 2, 3, 4))
            # print(a,b,c,d,e,sep=',')
            # a, b, c, d, e =-2,-2,4,1,-2
            f = (d * x + e) / (a * x ** 2 + b * x + c)
            f_prim = f.diff(x)
            f_bis = f_prim.diff(x)
            krytyczne = sp.solve(f_prim, x)
            asymptoty = sp.solve(a * x ** 2 + b * x + c, x)
            if ciekawe:
                if (all([el.is_real for el in krytyczne])
                        and all([10 * el == int(10 * el) for el in krytyczne])
                        and len(krytyczne) == 2  # trzech być nie może
                        and all([el.is_integer for el in asymptoty])
                        and len(asymptoty) == 2):
                    break
            else:
                if (all([el.is_real for el in krytyczne])
                        and all([10 * el == int(10 * el) for el in krytyczne])
                        and len(krytyczne) > 0  # trzech być nie może
                        and all([el.is_integer for el in asymptoty])
                        and len(asymptoty) >= 0):
                    break
                # print(f'rozwiazania {krytyczne}')
                # print(sp.latex(f))
        rosnaca = sp.solve(f_prim > 0, x)
        malejaca = sp.solve(f_prim < 0, x)
        # print(f, asymptoty, krytyczne, rosnaca, malejaca, sep='\n')
        if len(krytyczne) == 1:  # nie może być len(krytyczne) == 1 oraz len(asymptoty) == 0 lub len(asymptoty) == 2
            if a / d > 0:
                return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                        f'\t\\[\n'
                        f'\t\tf(x)={sp.latex(f)} \n'
                        f'\t\\]\n',
                        f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                        # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                        f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                        f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                        f'x \\in \\left( {sp.latex(rosnaca.args[0].args[0])}, {sp.latex(rosnaca.args[1].args[1])}  \\right)'
                        '$\\newline\n'
                        f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                        f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                        f'x \\in \\left( -\\infty, {sp.latex(malejaca.args[1].args[1])}  \\right)'
                        f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(malejaca.args[0].args[0])}, \\infty  \\right) $\\newline\n'
                        f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                        f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                        f' \\implies ' +
                        ('f_{\\min}' if f_bis.subs(x, krytyczne[0]) > 0 else 'f_{\\max}') +
                        f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))}$')
            else:
                return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                        f'\t\\[\n'
                        f'\t\tf(x)={sp.latex(f)} \n'
                        f'\t\\]\n',
                        f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                        # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                        f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                        f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                        f'x \\in \\left( -\\infty, {sp.latex(rosnaca.args[1].args[1])}  \\right)'
                        f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(rosnaca.args[0].args[0])}, \\infty    \\right)'
                        '$\\newline\n'
                        f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                        f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                        f'x \\in \\left( {sp.latex(malejaca.args[0].args[0])}, {sp.latex(malejaca.args[1].args[1])}  \\right) $\\newline\n'
                        f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                        f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                        f' \\implies ' +
                        ('f_{\\min}' if f_bis.subs(x, krytyczne[0]) > 0 else 'f_{\\max}') +
                        f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))}$')
        else:  # punktów stacjonarnych może być maksymalnie dwa
            # Todo: wyszły 4 przypadki - możliwe, że wyjdzie więcej - może się pojawić błąd
            if len(asymptoty) == 0:
                if a / d > 0:
                    return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                            f'\t\\[\n'
                            f'\t\tf(x)={sp.latex(f)} \n'
                            f'\t\\]\n',
                            f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                            # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                            f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                            f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                            f'x \\in \\left( {sp.latex(rosnaca.args[0].args[0])}, {sp.latex(rosnaca.args[1].args[1])}  \\right) $\\newline\n'
                            f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                            f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                            f'x \\in \\left( -\\infty, {sp.latex(malejaca.args[1].args[1])}  \\right)'
                            f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(malejaca.args[0].args[0])}, \\infty  \\right) $\\newline\n'
                            f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                            f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                            f' \\implies ' +
                            ('f_{\\min}' if f_bis.subs(x, krytyczne[0]) > 0 else 'f_{\\max}') +
                            f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))} \\textnormal{{ oraz }} ' +
                            ('f_{\\min}' if f_bis.subs(x, krytyczne[1]) > 0 else 'f_{\\max}') +
                            f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$')
                else:
                    return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                            f'\t\\[\n'
                            f'\t\tf(x)={sp.latex(f)} \n'
                            f'\t\\]\n',
                            f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                            # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                            f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                            f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                            f'x \\in \\left( -\\infty, {sp.latex(rosnaca.args[1].args[1])}  \\right)'
                            f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(rosnaca.args[0].args[0])}, \\infty    \\right)'
                            '$\\newline\n'
                            f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                            f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                            f'x \\in \\left( {sp.latex(malejaca.args[0].args[0])}, {sp.latex(malejaca.args[1].args[1])}  \\right) $\\newline\n'
                            f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                            f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                            f' \\implies ' +
                            ('f_{\\min}' if f_bis.subs(x, krytyczne[0]) > 0 else 'f_{\\max}') +
                            f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))} \\textnormal{{ oraz }} ' +
                            ('f_{\\min}' if f_bis.subs(x, krytyczne[1]) > 0 else 'f_{\\max}') +
                            f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$')
            if len(asymptoty) == 2:
                if a / d > 0:
                    return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                            f'\t\\[\n'
                            f'\t\tf(x)={sp.latex(f)} \n'
                            f'\t\\]\n',
                            f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                            # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                            f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                            f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                            f'x \\in \\left({sp.latex(rosnaca.args[0].args[1])},{sp.latex(rosnaca.args[2].args[1])}  \\right)'
                            f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(rosnaca.args[2].args[1])},{sp.latex(rosnaca.args[1].args[1])}  \\right) $\\newline\n'
                            f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                            f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                            f'x \\in \\left( -\\infty, {sp.latex(malejaca.args[1].args[1])}  \\right)'
                            f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(malejaca.args[2].args[0].args[0])}, {sp.latex(malejaca.args[2].args[1].args[1])}  \\right) '
                            f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(malejaca.args[0].args[0])}, \\infty  \\right) $\\newline\n'
                            f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                            f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                            f' \\implies ' +
                            ('f_{\\min}' if f_bis.subs(x, krytyczne[0]) > 0 else 'f_{\\max}') +
                            f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))} \\textnormal{{ oraz }} ' +
                            ('f_{\\min}' if f_bis.subs(x, krytyczne[1]) > 0 else 'f_{\\max}') +
                            f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$')
                else:
                    # print(a,b,c,d,e,sep=',')
                    return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                            f'\t\\[\n'
                            f'\t\tf(x)={sp.latex(f)} \n'
                            f'\t\\]\n',
                            f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                            # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                            f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                            f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                            f'x \\in \\left( -\\infty, {sp.latex(rosnaca.args[1].args[1])}  \\right)'
                            f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(rosnaca.args[2].args[0].args[0])}, {sp.latex(rosnaca.args[2].args[1].args[1])}  \\right) '
                            f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(rosnaca.args[0].args[0])}, \\infty  \\right) $\\newline\n'
                            f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                            f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                            f'x \\in \\left({sp.latex(malejaca.args[0].args[1])},{sp.latex(malejaca.args[2].args[1])}  \\right)'
                            f'\\textnormal{{ oraz }} x \\in \\left({sp.latex(malejaca.args[2].args[1])},{sp.latex(malejaca.args[1].args[1])}  \\right)  $\\newline\n'
                            f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                            f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                            f' \\implies ' +
                            ('f_{\\min}' if f_bis.subs(x, krytyczne[0]) > 0 else 'f_{\\max}') +
                            f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))} \\textnormal{{ oraz }} ' +
                            ('f_{\\min}' if f_bis.subs(x, krytyczne[1]) > 0 else 'f_{\\max}') +
                            f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$')
    if typ == 3:  # wielomian 3-go stopnia przez prostą
        def przedzialy(expr):  # nie podoba mi się to!!!
            nonlocal punkty
            if expr.func == sp.core.relational.StrictLessThan:
                if str(expr.args[0]) == 'x':
                    if -sp.oo not in punkty:
                        punkty.append(-sp.oo)
                    if expr.args[1] not in punkty:
                        punkty.append(expr.args[1])
                else:
                    punkty.append(sp.oo)
                    punkty.append(expr.args[0])
            if expr.func == sp.And:
                if expr.args[0].args[0] not in punkty:
                    punkty.append(expr.args[0].args[0])
                if expr.args[1].args[1] not in punkty:
                    punkty.append(expr.args[1].args[1])
            else:
                for arg in expr.args:
                    przedzialy(arg)
            return 0

        x = sp.Symbol('x', real=True)
        while True:
            b, c, d = [random.choice([-3, -2, -1, 1, 0, 0, 2, 3]) for _ in range(3)]
            a, e, g = [random.choice([-2, -1, 1, 2, 3]) for _ in range(3)]
            '''dla następnych danych są 3 ekstrema - wykluczamy to na razie'''
            # a, b, c, d, e, g = 2,2,-3,0,1,-2
            '''dla powyższych danych są 3 ekstrema - wykluczamy to na razie'''
            # a, b, c, d, e, g = -2, 0, 0, 0, 1, 2
            # a, b, c, d, e, g = -2, 0, 0, 0, -1, 1
            # a, b, c, d, e, g = 1, 0, 0, 0, -1, 2
            f = (a * x ** 3 + b * x ** 2 + c * x + d) / (e * x + g)
            f_prim = f.diff(x)
            krytyczne = sp.solve(f_prim, x)
            if (len(krytyczne) == 2
                    and all([el.is_real for el in krytyczne])
                    and all([(2 * el).is_integer for el in krytyczne])):
                # print([el.is_real for el in krytyczne])
                # print([2 * el.is_integer for el in krytyczne])
                break
        f_bis = f_prim.diff(x)
        rosnaca = sp.solve(f_prim > 0, x)
        malejaca = sp.solve(f_prim < 0, x)
        # print(f, krytyczne)
        # for st in krytyczne:
        #     print(f_bis.subs(x, st))

        if a / e > 0:
            # print(a, b, c, d, e, g, sep=',')
            ''' to mi sie strasznie nie podoba '''
            punkty = list()
            przedzialy(rosnaca)
            punkty_rosnaca = sorted(punkty)
            punkty = list()
            przedzialy(malejaca)
            punkty_malejaca = sorted(punkty)
            # print(punkty_rosnaca)
            # print(punkty_malejaca)
            if len(punkty_rosnaca) == 2:
                przedzial_rosnacy = f'\\left({sp.latex(punkty_rosnaca[0])},{sp.latex(sp.oo)}\\right)'
                przedzial_malejacy = (f'\\left({sp.latex(-sp.oo)},{sp.latex(punkty_malejaca[2])}\\right)'
                                      f' \\textnormal{{ oraz }} x \\in'
                                      f'  \\left({sp.latex(punkty_malejaca[2])},{sp.latex(punkty_malejaca[3])}\\right)')
            else:
                przedzial_rosnacy = (f'\\left({sp.latex(punkty_rosnaca[0])},{sp.latex(punkty_rosnaca[1])}\\right)'
                                     f' \\textnormal{{ oraz }} x \\in'
                                     f'  \\left({sp.latex(punkty_rosnaca[1])},{sp.latex(punkty_rosnaca[3])}\\right)')
                przedzial_malejacy = f'\\left({sp.latex(-sp.oo)},{sp.latex(punkty_malejaca[1])}\\right)'
            # print(przedzial_rosnacy)
            # print(przedzial_malejacy)
            return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                    f'\t\\[\n'
                    f'\t\tf(x)={sp.latex(f)} \n'
                    f'\t\\]\n',
                    f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                    # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                    f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                    f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                    f'x \\in {przedzial_rosnacy} $\\newline\n'
                    f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                    f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                    f'x \\in {przedzial_malejacy} $\\newline\n'
                    f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                    f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                    f' \\implies ' +
                    (
                        'f_{\\min}' + f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))}$' if f_bis.subs(
                            x,
                            krytyczne[
                                0]) > 0 else '') +
                    (
                        'f_{\\max}' + f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))}$' if f_bis.subs(
                            x,
                            krytyczne[
                                0]) < 0 else '') +
                    (
                        'f_{\\min}' + f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$' if f_bis.subs(
                            x,
                            krytyczne[
                                1]) > 0 else '') +
                    (
                        'f_{\\max}' + f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$' if f_bis.subs(
                            x,
                            krytyczne[
                                1]) < 0 else ''))
        else:
            # print(a, b, c, d, e, g, sep=',')
            ''' to mi sie strasznie nie podoba '''
            punkty = list()
            przedzialy(rosnaca)
            punkty_rosnaca = sorted(punkty)
            punkty = list()
            przedzialy(malejaca)
            punkty_malejaca = sorted(punkty)
            # print(punkty_rosnaca)
            # print(punkty_malejaca)
            if len(punkty_rosnaca) == 2:
                przedzial_rosnacy = f'\\left({sp.latex(-sp.oo)},{sp.latex(punkty_rosnaca[1])}\\right)'
                przedzial_malejacy = (f'\\left({sp.latex(punkty_malejaca[0])},{sp.latex(punkty_malejaca[1])}\\right)'
                                      f' \\textnormal{{ oraz }} x \\in'
                                      f'  \\left({sp.latex(punkty_malejaca[1])},{sp.latex(sp.oo)}\\right)')
            else:
                przedzial_rosnacy = (f'\\left({sp.latex(punkty_rosnaca[0])},{sp.latex(punkty_rosnaca[2])}\\right)'
                                     f' \\textnormal{{ oraz }} x \\in'
                                     f'  \\left({sp.latex(punkty_rosnaca[2])},{sp.latex(punkty_rosnaca[3])}\\right)')
                przedzial_malejacy = f'\\left({sp.latex(punkty_malejaca[0])},{sp.latex(sp.oo)}\\right)'
            # print(przedzial_rosnacy)
            # print(przedzial_malejacy)
            return (f'Zbadać monotoniczność i ekstrema funkcji\n'
                    f'\t\\[\n'
                    f'\t\tf(x)={sp.latex(f)} \n'
                    f'\t\\]\n',
                    f'$f\'(x)={sp.latex(sp.factor(f_prim))}$\\newline\n'
                    # f'$f\'\'(x)={sp.latex(sp.factor(f_bis))}$\\newline\n'
                    f'$f\'(x) > 0 \\textnormal{{ dla: }}'
                    f'{sp.latex(rosnaca)} \\implies f(x) \\textnormal{{ rośnie dla }} '
                    f'x \\in {przedzial_rosnacy}  $\\newline\n'
                    f'$f\'(x) < 0 \\textnormal{{ dla: }}'
                    f'{sp.latex(malejaca)} \\implies f(x) \\textnormal{{ maleje dla }} '
                    f'x \\in {przedzial_malejacy} $\\newline\n'
                    f'$f\'(x) = 0 \\textnormal{{ dla }} x \\in'
                    f' {sp.latex(krytyczne)}'.replace('[', '\\{').replace(']', '\\}') +
                    f' \\implies ' +
                    (
                        'f_{\\min}' + f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))}$' if f_bis.subs(
                            x,
                            krytyczne[
                                0]) > 0 else '') +
                    (
                        'f_{\\max}' + f'\\left({sp.latex(krytyczne[0])}\\right) = {sp.latex(f.subs(x, krytyczne[0]))}$' if f_bis.subs(
                            x,
                            krytyczne[
                                0]) < 0 else '') +
                    (
                        'f_{\\min}' + f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$' if f_bis.subs(
                            x,
                            krytyczne[
                                1]) > 0 else '') +
                    (
                        'f_{\\max}' + f'\\left({sp.latex(krytyczne[1])}\\right) = {sp.latex(f.subs(x, krytyczne[1]))}$' if f_bis.subs(
                            x,
                            krytyczne[
                                1]) < 0 else ''))


if __name__ == "__main__":
    start = time.time()
    gotowce = True
    os.chdir('..')  # by wczytywać z gotowca - inaczej problem ze ścieżkami!
    # polecenie, rozwiazanie = monotonicznosc(typ=3)
    polecenie, rozwiazanie = monotonicznosc(typ=3, gotowiec=gotowce)
    print(polecenie, '\n', rozwiazanie)
    print(f'czas generowania: {round(time.time() - start, 3)}s')



import os  # by sprawdzić istnienie pliku gotowców
import pickle
import random
import sys  # do czerwonych komunikatów: print('jakis tekst', file=sys.stderr )
import time  # dla testów wydajności
from pathlib import Path

import numpy as np
import sympy as sp

sciezka = str(Path(__file__).parent)


# Todo: dołożyć macierze zespolone jako osobne funkcje?

def rownanie_macierzowe():  # na 2000 losowań żadnego powtórzenia
    random.seed()
    rng = np.random.default_rng()
    # wspolczynniki = [-6,-5,-4,-3,-2,-1,2,3,4,5,6,7,8,9]
    while True:
        A = rng.integers(-1, 3, size=(2, 5))
        B = rng.integers(-2, 2, size=(2, 5))
        C = rng.integers(-1, 3, size=(5, 2))
        D = rng.integers(-2, 2, size=(5, 2))
        strona = random.choice(['l', 'p'])
        stala = random.choice([2, 3, 4])
        Ident = stala * np.identity(2)
        wyznacznik1 = random.choice([-3, -2, 2, 3, 4, 5, 6])
        wyznacznik2 = random.choice([-3, -2, 2, 3, 4, 5])
        if abs(np.linalg.det(A @ B.transpose()) - wyznacznik1) < 0.1 and abs(
                np.linalg.det(C.transpose() @ D - Ident) - wyznacznik2) < 0.1:
            break
    # print('A=',(sp.Array(A)))
    # print('B=',(sp.Array(B)))
    # print('C=',(sp.Array(C)))
    # print('D=',(sp.Array(D)))
    if strona == 'p':
        rownanie = ('\t\\[\n'
                    f'\t\t{sp.latex(sp.Array(A))}\\cdot \n'
                    f'\t\t{sp.latex(sp.Array(B))}^T + \n'
                    f'\t\t{stala}X=\n'
                    f'\t\t{sp.latex(sp.Array(C))}^T \\cdot\n'
                    f'\t\t{sp.latex(sp.Array(D))} \\cdot X\n'
                    f'\t\\]')
        # pomysł ze strony https://stackoverflow.com/questions/43804701/round-floats-within-an-expression
        # do wyeliminowania wyników postaci 3.342432*10^(-16) i 0.999999999999997
        Wynik = sp.Matrix(
            ((np.linalg.inv(C.transpose() @ D - Ident)) @ A @ B.transpose() * np.linalg.det(C.transpose() @ D - Ident)))
        for a in sp.preorder_traversal(Wynik):
            if isinstance(a, sp.Float):
                Wynik = Wynik.subs(a, round(a, 10))
        rozwiazanie = (
            f'$ {sp.latex(sp.Array(A @ B.transpose()))} + \n'
            f'\t{stala}X=\n'
            f'\t{sp.latex(sp.Array(C.transpose() @ D))} \\cdot X, \\quad \n'
            f'\t{sp.latex(sp.Array(A @ B.transpose()))} = \n'
            f'\t{sp.latex(sp.Array((C.transpose() @ D) - Ident))} \\cdot X $ \\\\ \n'
            f'\t$X=\\frac{{1}}{{{int(np.rint(np.linalg.det(C.transpose() @ D - Ident)))}}}'
            # f'{sp.latex(sp.Matrix(((np.linalg.inv(C.transpose() @ D - Ident)) @ A @ B.transpose() * np.linalg.det(C.transpose() @ D - Ident))))}.'
            f'{sp.latex(Wynik)}.$')
        rozwiazanie = rozwiazanie.replace('.0', '')  # liczby w macierzach są całkowite i nie chcę wyświetlać .0
    else:
        rownanie = ('\t\\[\n'
                    f'\t\t{sp.latex(sp.Array(A))}\\cdot \n'
                    f'\t\t{sp.latex(sp.Array(B))}^T + \n'
                    f'\t\t{stala}X=\n'
                    f'\t\tX \\cdot {sp.latex(sp.Array(C))}^T \\cdot\n'
                    f'\t\t{sp.latex(sp.Array(D))} \n'
                    f'\t\\]')
        # rownanie = '        $$' + str(sp.latex(sp.Array(A))) + '\\cdot \n' + \
        #            '        ' + str(sp.latex(sp.Array(B))) + '^T +\n ' + \
        #            '        ' + str(stala) + ' X=X \\cdot' + \
        #            '        ' + str(sp.latex(sp.Array(C))) + '^T \\cdot\n' + \
        #            '        ' + str(sp.latex(sp.Array(D))) + '$$\n'
        Wynik = sp.Matrix(
            ((A @ B.transpose() @ np.linalg.inv(C.transpose() @ D - Ident)) * np.linalg.det(C.transpose() @ D - Ident)))
        for a in sp.preorder_traversal(Wynik):
            if isinstance(a, sp.Float):
                Wynik = Wynik.subs(a, round(a, 10))
        rozwiazanie = (
            f'$ {sp.latex(sp.Array(A @ B.transpose()))} + \n'
            f'\t{stala}X=\n'
            f'\tX \\cdot {sp.latex(sp.Array(C.transpose() @ D))} , \\quad \n'
            f'\t{sp.latex(sp.Array(A @ B.transpose()))} = \n'
            f'\tX \\cdot {sp.latex(sp.Array((C.transpose() @ D) - Ident))} $ \\\\ \n'
            f'\t$X=\\frac{{1}}{{{int(np.rint(np.linalg.det(C.transpose() @ D - Ident)))}}}'
            # f'{sp.latex(sp.Matrix(((A @ B.transpose() @ np.linalg.inv(C.transpose() @ D - Ident)) * np.linalg.det(C.transpose() @ D - Ident))))}.\n'
            f'{sp.latex(Wynik)}.$')
        rozwiazanie = rozwiazanie.replace('.0', '')  # liczby w macierzach są całkowite i nie chcę wyświetlać .0
    return ('Rozwiązać równanie:\n'
            f'{rownanie}',
            f'{rozwiazanie}')


def wyznacznik_parametr(wymiar: int = 3, gotowiec: bool = False):  # na 1000 losowań nie było żadnej powtórki

    if gotowiec is True:
        if wymiar == 4:
            if os.path.isfile(sciezka + '//gotowe//wyznacznik_parametr_wymiar_4.pickle'):  # 10000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//wyznacznik_parametr_wymiar_4.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
        if wymiar == 3:
            if os.path.isfile(sciezka + '//gotowe//wyznacznik_parametr_wymiar_3.pickle'):  # 10000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//wyznacznik_parametr_wymiar_3.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
    # to nie musi być w else, bo wcześniejsze warunku w przypadku sukcesu konczą funkcje returnem
    x = sp.symbols('x', real=True)
    while True:
        M = sp.randMatrix(wymiar, wymiar, -4, 4)
        licznik_zer = 0
        for ii in range(wymiar):
            for jj in range(wymiar):
                if M[ii, jj] == 0:
                    licznik_zer += 1
        for k in range(wymiar + 1):
            wiersz = random.randint(0, wymiar - 1)
            kol = random.randint(0, wymiar - 1)
            M[wiersz, kol] += x
        wyznacznik = sp.det(M)
        pierwiastki = sp.solve(wyznacznik)
        if (
                2 <= sp.degree(wyznacznik) <= max(2, min(wymiar - 1, 3))  # nie za duże wielomiany
                and 0 < len(pierwiastki) <= max(2, wymiar - 1)  # czy to gwarantuje różne pierwiastki?
                and wyznacznik.coeff(x, sp.degree(wyznacznik)) in {-3, -2, -1, 2, 3}
                and abs(wyznacznik.coeff(x, 0)) < 50  # by wyznacznik nie miał za dużych współczynników
                and licznik_zer < wymiar - 1
                and all(pierwiastek.is_integer for pierwiastek in pierwiastki)
        ):
            break
    return ('Dla jakich rzeczywistych wartości parametru $x$ wyznacznik macierzy $A$ jest różny od zera?\n'
            f'\t\\[\n'
            f'\t\t\\textnormal{{A=}}\n'
            f'\t\t{sp.latex(M)}\n'
            f'\t\\]',
            f'$\\det A='
            f'{sp.latex(wyznacznik)} \\neq 0, \\quad ' +
            ''.join(f'x\\neq {pierwiastek}, \\ ' for pierwiastek in pierwiastki) +
            f'$')


def macierz_odwrotna_parametr(wymiar: int = 3, gotowiec: bool = False):
    if gotowiec is True:
        if wymiar == 3:
            if os.path.isfile(
                    sciezka + '//gotowe//macierz_odwrotna_parametr_wymiar_3.pickle'):  # 10000/10000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//macierz_odwrotna_parametr_wymiar_4.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
        if wymiar == 4:
            if os.path.isfile(sciezka + '//gotowe//macierz_odwrotna_parametr_wymiar_4.pickle'):  # 1000 różnych gotowych
                gotowe = pickle.load(open(sciezka + '//gotowe//macierz_odwrotna_parametr_wymiar_4.pickle', 'rb'))
                return gotowe[random.randint(0, len(gotowe)) - 1]
            else:
                print('Brak gotowca do tego typu', file=sys.stderr)
    # to nie musi być w else, bo wcześniejsze warunku w przypadku sukcesu konczą funkcje returnem
    x = sp.symbols('x', real=True)
    while True:
        M = sp.randMatrix(wymiar, wymiar, -4, 4)
        licznik_zer = 0
        for ii in range(wymiar):
            for jj in range(wymiar):
                if M[ii, jj] == 0:
                    licznik_zer += 1
        for k in range(4):
            wiersz = random.randint(0, wymiar - 1)
            kol = random.randint(0, wymiar - 1)
            M[wiersz, kol] += x
        wyznacznik = sp.det(M)
        pierwiastki = sp.solve(wyznacznik)

        if (
                sp.degree(wyznacznik) == 2
                and len(pierwiastki) == 2
                and wyznacznik.coeff(x, 2) in {-3, -2, -1, 2, 3}
                and licznik_zer < wymiar - 1
                and all(pierwiastek.is_integer for pierwiastek in pierwiastki)
        ):
            podstaw = random.choice(list({-2, -1, 2, 3} - set(pierwiastki)))
            wyznacznik_podstaw = wyznacznik.subs(x, podstaw)
            # print(pierwiastki)
            # print(wyznacznik_podstaw, licznik_zer)
            if 1 < abs(wyznacznik_podstaw) < 3 ** wymiar:  # trochę duży.
                break

    # print(podstaw)
    # print(wyznacznik_podstaw)
    N = M.subs(x, podstaw)
    return ('Dla jakich rzeczywistych wartości parametru $x$ macierz $A$ posiada odwrotność?\n'
            f'\t\\[\n'
            f'\t\t\\textnormal{{A=}}{sp.latex(M)}\n'
            f'\t\\]\n'
            f'Wyznaczyć macierz odwrotną dla $x={podstaw}.$ Wykonać sprawdzenie.\\\\',
            f'$\\det A={sp.latex(wyznacznik)}\\neq 0, \\quad \n'
            f'\tx\\neq {pierwiastki[0]}\\ \\textnormal{{oraz}} \\ x\\neq {pierwiastki[1]},$\\\\\n'
            f'\t$A({podstaw})= {sp.latex(M.subs(x, podstaw))},\\ \n'
            f'\t\\det A({podstaw})={wyznacznik_podstaw},\\ \n'
            f'\tA^{{-1}}={sp.latex(sp.sympify(1 / wyznacznik_podstaw))}'
            f'{sp.latex((N.inv()) * wyznacznik_podstaw)}.$')


def wartosci_wlasne(wymiar: int = 3, zespolone: bool = False):  # na 10000 losowań 50 powtórzeń
    liczby = (-2, -1, 0, 1, 2, 3)
    while True:
        # wymiar = 4
        # zespolone = True

        M = sp.Matrix(wymiar, wymiar, [random.choice(liczby) for _ in range(wymiar ** 2)])
        N = sp.matrix2numpy(M, dtype=float)
        w, v = np.linalg.eig(N)

        if (wymiar == 2):
            if zespolone is False:
                rzeczywiste_ww = True
                for i in w:
                    if abs(np.imag(i)) > 0.01:
                        rzeczywiste_ww = False
                if rzeczywiste_ww is True:
                    break
            else:
                zespolone_ww = True
                for i in w:
                    if abs(np.imag(i)) < 0.01:
                        zespolone_ww = False
                if zespolone_ww is True:
                    break
        if wymiar == 3:
            if zespolone is False:
                rzeczywiste_ww = True
                for i in w:
                    if abs(np.imag(i)) > 0.01:
                        rzeczywiste_ww = False
                    if np.around(np.real(i), 0) != np.around(np.real(i), 2):
                        rzeczywiste_ww = False
                if rzeczywiste_ww is True:
                    break
            else:
                zespolone_ww = False
                for i in w:
                    if abs(np.imag(i)) > 0.01:
                        zespolone_ww = True
                calkowite = False
                if zespolone_ww is True:
                    calkowite = True
                    for i in w:
                        if (np.around(np.real(i), 0) != np.around(np.real(i), 2)
                                or np.around(np.imag(i), 0) != np.around(np.imag(i), 2)):
                            calkowite = False
                if zespolone_ww is True and calkowite is True:
                    break
        if wymiar >= 4:
            ladne_ww = True
            for i in w:
                if not (np.around(np.real(i), 0) == np.around(np.real(i), 2)
                        and np.around(np.imag(i), 0) == np.around(np.imag(i), 2)
                        and abs(np.imag(i)) < 0.01):
                    ladne_ww = False
            if ladne_ww is True:
                break

    return (f'Wyznaczyć wartości własne i wektory własne macierzy \n'
            f'\t\\[\n'
            f'\t\t\\textnormal{{A=}} {sp.latex(M)}\n'
            f'\t\\]',
            f'$\\det\\left(\\lambda \mathbb{{I}} - A \\right) = {sp.latex(M.charpoly().as_expr())}$, \\quad \n'  # znak: (A-lambda*I)
            f'Wartości własne:$ {sp.latex(M.eigenvals())}, $\\\\\n'
            f'\t\\parbox{{4em}}{{Wektory\\\\własne:}} ${sp.latex(M.eigenvects())}$')


# Tdoo: odpowiedź wektory włąsne w dwóch linijkach bo 4x4 jest za szerokie czasami.
def diagonalizacja_macierzy(wymiar: int = 3):  # na 10000 losowań 50 powtórzeń
    liczby = (-2, -1, 0, 1, 2, 3)
    while True:
        M = sp.Matrix(wymiar, wymiar, [random.choice(liczby) for _ in range(wymiar ** 2)])
        N = sp.matrix2numpy(M, dtype=float)
        w, v = np.linalg.eig(N)
        rzeczywiste_ww = True
        for i in w:
            if abs(np.imag(i)) > 0.01:
                rzeczywiste_ww = False
            if np.around(np.real(i), 0) != np.around(np.real(i), 2):
                rzeczywiste_ww = False
        # print(n, end = ',')
        if rzeczywiste_ww is True:
            break
    if M.is_diagonalizable():
        P, D = M.diagonalize()
        return (f'Przeprowadzić diagonalizację macierzy (jeśli możliwa) \n'
                f'\t\\[\n'
                f'\t\t\\textnormal{{A=}} {sp.latex(M)}\n'
                f'\t\\]',
                f'$\\det\\left(\\lambda \mathbb{{I}} - A \\right) = {sp.latex(M.charpoly().as_expr())}$, \\quad \n'
                f'Wartości własne:$ {sp.latex(M.eigenvals())}, $\\\\\n'
                f'\\parbox{{4em}}{{Wektory\\\\własne:}} ${sp.latex(M.eigenvects())}$,\\\\\n'
                f'$A = P\\,D\\,P^{{-1}}: \\quad P={sp.latex(P)},$ \\quad \n'
                f'$D={sp.latex(D)}$')
    else:
        return (f'Przeprowadzić diagonalizację macierzy (jeśli możliwa) \n'
                f'\t\\[\n'
                f'\t\t\\textnormal{{A=}} {sp.latex(M)}\n'
                f'\t\\]',
                f'$\\det\\left(\\lambda \mathbb{{I}} - A \\right) = {sp.latex(M.charpoly().as_expr())}$, \\quad \n'
                f'Wartości własne:$ {sp.latex(M.eigenvals())}, $\\\\\n'
                f'\\parbox{{4em}}{{Wektory\\\\własne:}} ${sp.latex(M.eigenvects())}$,\\\\\n'
                f'Macierz nie jest diagonalizowalna.')


def diagonalizacja_macierzy_z_wielokrotnym_wartosciami_wlasnymi(wymiar: int = 3):
    liczby = (-2, -1, 0, 1, 2, 3)

    while True:
        M = sp.Matrix(wymiar, wymiar, [random.choice(liczby) for _ in range(wymiar ** 2)])
        values = [*M.eigenvals().values()]
        if max(values) > 1:
            try:
                P, D = M.diagonalize()
            except Exception:
                pass
            else:
                break
    return (f'Przeprowadzić diagonalizację macierzy\n'
            f'\t\\[\n'
            f'\t\t\\textnormal{{A=}} {sp.latex(M)}\n'
            f'\t\\]',
            f'$\\det\\left(\\lambda \mathbb{{I}} - A \\right) = {sp.latex(M.charpoly().as_expr())}$, \\quad \n'
            f'Wartości własne:$ {sp.latex(M.eigenvals())}, $\\\\\n'
            f'\\parbox{{4em}}{{Wektory\\\\własne:}} ${sp.latex(M.eigenvects())}$,\\\\\n'
            f'$A = P\\,D\\,P^{{-1}}: \\quad P={sp.latex(P)},$ \\quad \n'
            f'$D={sp.latex(D)}$')


def rzad_macierzy():
    liczby = (-2, -1, 0, 1, 2, 3)
    wymiar = random.choice((4, 5, 6))
    licznik = 0
    los = np.random.choice(([0] * 4 + [1] * 1))  # pozwól na rząd równy wymiarowi macierzy — ale nie za często
    while True:
        licznik += 1
        M = sp.Matrix(wymiar, 10 - wymiar, [random.choice(liczby) for _ in range(wymiar * (10 - wymiar))])
        N = N = sp.matrix2numpy(M, dtype=np.int32)  # bez astype nie będzie sprawdzał poprawnie logiki macierzy!!!
        za_duzo_zer = (~np.any(N, axis=0)).any() or (~np.any(N, axis=1)).any()
        """ Następne robi to samo co ostatnie dwie linijki ale jakoś strasznie długo? 
        # m, n = np.shape(M)
        # za_duzo_zer = False # unikamy wierszy lub kolumn pełnych zer
        # for i in range(n):
        #     if np.count_nonzero(M[:, i]) <= 1:
        #         za_duzo_zer = True
        # for i in range(m):
        #     if np.count_nonzero(M[i, :]) <= 1:
        #         za_duzo_zer = True
        """
        if M.rank() < min(wymiar, 10 - wymiar) + los and not za_duzo_zer:
            break
    # print(licznik)
    # print(len(M) - np.count_nonzero(M))

    return ('Wyznaczyć rząd macierzy A\n'
            f'\t\\[\n'
            f'\t\t\\textnormal{{A=}} {sp.latex(M)}\n'
            f'\t\\]',
            f'$R(A)= {M.rank()}$')


if __name__ == "__main__":  # to się uruchamia tylko, gdy plik jest uruchamiany jako program, a nie ładowany jako moduł
    gotowce = True
    os.chdir('..')  # by wczytywać z gotowca - inaczej problem ze ścieżkami!
    start_time = time.time()

    polecenie, rozwiazanie = wyznacznik_parametr(wymiar=3, gotowiec=gotowce)
    # polecenie, rozwiazanie = wyznacznik_parametr(wymiar=random.choice(([2] * 1) + ([3] * 7) + ([4] * 0)))
    # polecenie, rozwiazanie = rownanie_macierzowe()
    # polecenie, rozwiazanie = wyznacznik_parametr(wymiar=random.choice(([2] * 0) + ([3] * 0) + ([4] * 1)),
    #                                                    gotowiec=True)
    # polecenie, rozwiazanie = wartosci_wlasne(wymiar=random.choice(([2] * 1) + ([3] * 7) + ([4] * 1)),
    #                                          zespolone=random.choice((True, False)))
    # polecenie, rozwiazanie = diagonalizacja_macierzy(wymiar=random.choice(([2]*4) + ([3]*6) + ([4]*1)))
    # polecenie, rozwiazanie = diagonalizacja_macierzy_z_wielokrotnym_wartosciami_wlasnymi(wymiar=3)
    # polecenie, rozwiazanie = rzad_macierzy()

    print(polecenie, '\n', rozwiazanie)
    print("Czas generowania zestawów --- %s seconds ---" % (time.time() - start_time))
